#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkimageviewer.py,v $
## Language:  Python
## Date:      $Date: 2006/05/26 12:35:13 $
## Version:   $Revision: 1.8 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
import sys

from vmtk import vtkvmtk
from vmtk import vmtkrenderer
from vmtk import pypes


class vmtkImageViewer(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Image = None
        self.vmtkRenderer = None
        self.OwnRenderer = 0
        self.Display = 1
        self.ArrayName = ''

        self.Picker = None
        self.PlaneWidgetX = None
        self.PlaneWidgetY = None
        self.PlaneWidgetZ = None

        self.Margins = 0
        self.TextureInterpolation = 1
        self.ContinuousCursor = 0
        self.WindowLevel = [0.0, 0.0]

        self.SetScriptName('vmtkimageviewer')
        self.SetScriptDoc('display a 3D image')
        self.SetInputMembers([
            ['Image','i','vtkImageData',1,'','the input image','vmtkimagereader'],
            ['ArrayName','array','str',1,'','name of the array to display'],
            ['vmtkRenderer','renderer','vmtkRenderer',1,'','external renderer'],
            ['WindowLevel','windowlevel','float',2,'','the window/level for displaying the image'],
            ['Display','display','bool',1,'','toggle rendering'],
            ['Margins','margins','bool',1,'','toggle margins for tilting image planes'],
            ['TextureInterpolation','textureinterpolation','bool',1,'','toggle interpolation of graylevels on image planes'],
            ['ContinuousCursor','continuouscursor','bool',1,'','toggle use of physical continuous coordinates for the cursor']
            ])
        self.SetOutputMembers([
            ['Image','o','vtkImageData',1,'','the output image','vmtkimagewriter'],
            ['PlaneWidgetX','xplane','vtkImagePlaneWidget',1,'','the X image plane widget'],
            ['PlaneWidgetY','yplane','vtkImagePlaneWidget',1,'','the Y image plane widget'],
            ['PlaneWidgetZ','zplane','vtkImagePlaneWidget',1,'','the Z image plane widget']
            ])

    def CharCallback(self, obj):
        return

    def BuildView(self):

        if not self.vmtkRenderer:
            self.vmtkRenderer = vmtkrenderer.vmtkRenderer()
            self.vmtkRenderer.Initialize()
            self.OwnRenderer = 1

        self.vmtkRenderer.RegisterScript(self)

        if (self.ArrayName != ''):
            self.Image.GetPointData().SetActiveScalars(self.ArrayName)
        wholeExtent = self.Image.GetExtent()

        if self.Picker == None:
            self.Picker = vtk.vtkCellPicker()

        if self.PlaneWidgetX == None:
            self.PlaneWidgetX = vtkvmtk.vtkvmtkImagePlaneWidget()

        if self.PlaneWidgetY == None:
            self.PlaneWidgetY = vtkvmtk.vtkvmtkImagePlaneWidget()

        if self.PlaneWidgetZ == None:
            self.PlaneWidgetZ = vtkvmtk.vtkvmtkImagePlaneWidget()

        self.Picker.SetTolerance(0.005)

        self.PlaneWidgetX.SetResliceInterpolateToLinear()
        self.PlaneWidgetX.SetTextureInterpolate(self.TextureInterpolation)
        self.PlaneWidgetX.SetUseContinuousCursor(self.ContinuousCursor)
        self.PlaneWidgetX.SetInputData(self.Image)
        self.PlaneWidgetX.SetPlaneOrientationToXAxes()
        self.PlaneWidgetX.SetSliceIndex(wholeExtent[0])
        if self.vmtkRenderer.Annotations:
            self.PlaneWidgetX.DisplayTextOn()
        else:
            self.PlaneWidgetX.DisplayTextOff()
        self.PlaneWidgetX.SetPicker(self.Picker)
        self.PlaneWidgetX.KeyPressActivationOff()
        self.PlaneWidgetX.SetInteractor(self.vmtkRenderer.RenderWindowInteractor)
        if self.Margins:
            self.PlaneWidgetX.SetMarginSizeX(0.05)
            self.PlaneWidgetX.SetMarginSizeY(0.05)
        else:
            self.PlaneWidgetX.SetMarginSizeX(0.0)
            self.PlaneWidgetX.SetMarginSizeY(0.0)
        if self.WindowLevel[0] != 0.0:
            self.PlaneWidgetX.SetWindowLevel(self.WindowLevel[0],self.WindowLevel[1])
        self.PlaneWidgetX.On()

        self.PlaneWidgetY.SetResliceInterpolateToLinear()
        self.PlaneWidgetY.SetTextureInterpolate(self.TextureInterpolation)
        self.PlaneWidgetY.SetUseContinuousCursor(self.ContinuousCursor)
        self.PlaneWidgetY.SetInputData(self.Image)
        self.PlaneWidgetY.SetPlaneOrientationToYAxes()
        self.PlaneWidgetY.SetSliceIndex(wholeExtent[2])
        if self.vmtkRenderer.Annotations:
            self.PlaneWidgetY.DisplayTextOn()
        else:
            self.PlaneWidgetY.DisplayTextOff()
        self.PlaneWidgetY.SetPicker(self.Picker)
        self.PlaneWidgetY.KeyPressActivationOff()
        self.PlaneWidgetY.SetLookupTable(self.PlaneWidgetX.GetLookupTable())
        self.PlaneWidgetY.SetInteractor(self.vmtkRenderer.RenderWindowInteractor)
        if self.Margins:
            self.PlaneWidgetY.SetMarginSizeX(0.05)
            self.PlaneWidgetY.SetMarginSizeY(0.05)
        else:
            self.PlaneWidgetY.SetMarginSizeX(0.0)
            self.PlaneWidgetY.SetMarginSizeY(0.0)
        if self.WindowLevel[0] != 0.0:
            self.PlaneWidgetY.SetWindowLevel(self.WindowLevel[0],self.WindowLevel[1])
        self.PlaneWidgetY.On()

        self.PlaneWidgetZ.SetResliceInterpolateToLinear()
        self.PlaneWidgetZ.SetTextureInterpolate(self.TextureInterpolation)
        self.PlaneWidgetZ.SetUseContinuousCursor(self.ContinuousCursor)
        self.PlaneWidgetZ.SetInputData(self.Image)
        self.PlaneWidgetZ.SetPlaneOrientationToZAxes()
        self.PlaneWidgetZ.SetSliceIndex(wholeExtent[4])
        if self.vmtkRenderer.Annotations:
            self.PlaneWidgetZ.DisplayTextOn()
        else:
            self.PlaneWidgetZ.DisplayTextOff()
        self.PlaneWidgetZ.SetPicker(self.Picker)
        self.PlaneWidgetZ.KeyPressActivationOff()
        self.PlaneWidgetZ.SetLookupTable(self.PlaneWidgetX.GetLookupTable())
        self.PlaneWidgetZ.SetInteractor(self.vmtkRenderer.RenderWindowInteractor)
        if self.Margins:
            self.PlaneWidgetZ.SetMarginSizeX(0.05)
            self.PlaneWidgetZ.SetMarginSizeY(0.05)
        else:
            self.PlaneWidgetZ.SetMarginSizeX(0.0)
            self.PlaneWidgetZ.SetMarginSizeY(0.0)
        if self.WindowLevel[0] != 0.0:
            self.PlaneWidgetZ.SetWindowLevel(self.WindowLevel[0],self.WindowLevel[1])
        self.PlaneWidgetZ.On()

        if (self.Display == 1):
            self.vmtkRenderer.Render()

        if self.OwnRenderer:
            self.vmtkRenderer.Deallocate()

    def Execute(self):

        if (self.Image == None) & (self.Display == 1):
            self.PrintError('Error: no Image.')

        self.BuildView()


if __name__=='__main__':
    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



## Program: VMTK
## Language:  Python
## Date:      February 2, 2018
## Version:   1.4

##   Copyright (c) Richard Izzo, Luca Antiga, All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## Note: this code was contributed by
##       Richard Izzo (Github @rlizzo)
##       University at Buffalo

import pytest
import vmtk.vmtkbranchclipper as branchclipper
from vtk.numpy_interface import dataset_adapter as dsa
import numpy as np


def test_clip_returns_3_groups(aorta_centerline_branches, aorta_surface):
    clipper = branchclipper.vmtkBranchClipper()
    clipper.Centerlines = aorta_centerline_branches
    clipper.Surface = aorta_surface
    clipper.RadiusArrayName = 'MaximumInscribedSphereRadius'
    clipper.BlankingArrayName = 'Blanking'
    clipper.GroupIdsArrayName = 'GroupIds'
    clipper.Execute()

    wrapedclip = dsa.WrapDataObject(clipper.Surface)
    uniqueGroups = np.unique(np.array(wrapedclip.PointData['GroupIds'].tolist()))

    assert np.allclose(uniqueGroups, np.array([0, 2, 3])) == True


@pytest.mark.parametrize("groupids,paramid", [
    ([0], '0'),
    ([2], '2'),
    ([3], '3'),
    ([0, 2], '02'),
    ([2, 3], '23'),
    ([0, 2, 3], '023')
])
def test_clip_groupids_surface_returns_as_expected(aorta_centerline_branches, aorta_surface,
                                                  groupids, paramid, compare_surfaces):
    name = __name__ + '_test_clip_group_' + paramid + '_surface_returns_as_expected.vtp'
    clipper = branchclipper.vmtkBranchClipper()
    clipper.Centerlines = aorta_centerline_branches
    clipper.Surface = aorta_surface
    clipper.RadiusArrayName = 'MaximumInscribedSphereRadius'
    clipper.BlankingArrayName = 'Blanking'
    clipper.GroupIdsArrayName = 'GroupIds'
    clipper.GroupIds = groupids
    clipper.Execute()

    assert compare_surfaces(clipper.Surface, name) == True


@pytest.mark.parametrize("groupids,paramid", [
    ([0], '0'),
    ([2], '2'),
    ([3], '3')
])
def test_clip_groupids_with_insideout_surface_returns_as_expected(aorta_centerline_branches, aorta_surface,
                                                                 groupids, paramid, compare_surfaces):
    name = __name__ + '_test_clip_groupids_' + paramid + '_with_insideout_surface_returns_as_expected.vtp'
    clipper = branchclipper.vmtkBranchClipper()
    clipper.Centerlines = aorta_centerline_branches
    clipper.Surface = aorta_surface
    clipper.RadiusArrayName = 'MaximumInscribedSphereRadius'
    clipper.BlankingArrayName = 'Blanking'
    clipper.GroupIdsArrayName = 'GroupIds'
    clipper.GroupIds = groupids
    clipper.InsideOut = 1
    clipper.Execute()

    assert compare_surfaces(clipper.Surface, name, method="addpointarray", arrayname="ClippingArray") == True



## Program: VMTK
## Language:  Python
## Date:      February 2, 2018
## Version:   1.4

##   Copyright (c) Richard Izzo, Luca Antiga, All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## Note: this code was contributed by
##       Richard Izzo (Github @rlizzo)
##       University at Buffalo

import pytest
import vmtk.vmtkbranchextractor as branchextractor
from vtk.numpy_interface import dataset_adapter as dsa
import numpy as np


@pytest.fixture(scope='module')
def centerline_branches(aorta_centerline):
    extractor = branchextractor.vmtkBranchExtractor()
    extractor.Centerlines = aorta_centerline
    extractor.RadiusArrayName = 'MaximumInscribedSphereRadius'
    extractor.Execute()

    return extractor.Centerlines


def test_centerline_surface_distance(centerline_branches, compare_centerlines):
    name = __name__ + '_test_centerline_surface_distance.vtp'
    assert compare_centerlines(centerline_branches, name) == True


def test_number_of_lines(centerline_branches):
    assert centerline_branches.GetNumberOfLines() == 6


def test_number_of_cells(centerline_branches):
    assert centerline_branches.GetNumberOfCells() == 6


def test_number_of_points(centerline_branches):
    assert centerline_branches.GetNumberOfPoints() == 417


def test_cell_data_array_names(centerline_branches):
    expected_array_names = [
        "CenterlineIds",
        "TractIds",
        "Blanking",
        "GroupIds",
    ]
    cell_data = centerline_branches.GetCellData()
    array_names = [cell_data.GetArrayName(idx) for idx in range(cell_data.GetNumberOfArrays())]
    assert sorted(expected_array_names) == sorted(array_names)


@pytest.mark.parametrize("expectedvalue,paramname", [
    ([0, 0, 0, 1, 1, 1], "CenterlineIds"),
    ([0, 1, 2, 0, 1, 2], "TractIds"),
    ([0, 1, 0, 0, 1, 0], "Blanking"),
    ([0, 1, 2, 0, 1, 3], "GroupIds")
])
def test_cell_data_array_values(centerline_branches, expectedvalue, paramname):
    centerline_branches_wrapped = dsa.WrapDataObject(centerline_branches)
    assert centerline_branches_wrapped.CellData.GetArray(paramname).tolist() == expectedvalue


@pytest.mark.parametrize("expectedvalue,paramid", [
    (78, 0),
    (21, 1),
    (116, 2),
    (73, 3),
    (26, 4),
    (103, 5)
])
def test_number_of_points_per_cell(centerline_branches, expectedvalue, paramid):
    bcx = centerline_branches.GetCell(paramid)
    numberOfPoints = bcx.GetNumberOfPoints()

    assert numberOfPoints == expectedvalue


@pytest.mark.parametrize("expectedvalue,paramid", [
    (0, 0),
    (78, 1),
    (99, 2),
    (215, 3),
    (288, 4),
    (314, 5)
])
def test_cell_data_pointId_start_indices(centerline_branches, expectedvalue, paramid):
    bcx = centerline_branches.GetCell(paramid)
    pointIdStart = bcx.GetPointId(0)

    assert pointIdStart == expectedvalue


@pytest.mark.parametrize("expectedvalue,numberofpoints,paramid", [
    (77, 78, 0),
    (98, 21, 1),
    (214, 116, 2),
    (287, 73, 3),
    (313, 26, 4),
    (416, 103, 5)
])
def test_cell_data_pointId_end_indices(centerline_branches, expectedvalue, numberofpoints, paramid):
    bcx = centerline_branches.GetCell(paramid)
    pointIdEnd = bcx.GetPointId(numberofpoints - 1)

    assert pointIdEnd == expectedvalue


@pytest.mark.parametrize("pointidstart,numberofpoints,expectedlocationstart,expectedlocationend,paramid", [
    (0, 78, np.array([ 222.09629822, 175.8699646, 21.67310715]), np.array([ 222.55067444, 140.65298462, 23.77128601]), 0),
    (77, 21, np.array([222.55067444, 140.65298462, 23.77128601]), np.array([225.06373596, 134.70544434, 23.55709076]), 1),
    (99, 116, np.array([225.06373596, 134.70544434, 23.55709076]), np.array([234.35397339, 101.31446838, 28.98675537]), 2),
    (215, 73, np.array([222.09629822, 175.8699646, 21.67310715]), np.array([220.89038086, 140.92050171, 23.99687004]), 3),
    (288, 26, np.array([220.89038086, 140.92050171, 23.99687004]), np.array([218.53140259, 134.76171875, 24.90955925]), 4),
    (314, 103, np.array([218.53140259, 134.76171875, 24.90955925]), np.array([210.20037842, 103.04645538, 31.68768501]), 5)
])
def test_cell_data_point_start_and_end_xyz_locations(centerline_branches, pointidstart, numberofpoints,
                                                     expectedlocationstart, expectedlocationend, paramid):
    bcx = centerline_branches.GetCell(paramid)
    bw = dsa.WrapDataObject(centerline_branches)

    pointIdEnd = bcx.GetPointId(numberofpoints - 1)
    pointLocationEnd = bw.Points[pointIdEnd]
    pointLocationStart = bw.Points[pointidstart]

    assert np.allclose(np.array(pointLocationStart), expectedlocationstart) == True
    assert np.allclose(np.array(pointLocationEnd), expectedlocationend) == True



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkimagefeaturecorrection.py,v $
## Language:  Python
## Date:      $Date: 2006/07/17 09:53:14 $
## Version:   $Revision: 1.8 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
import sys

from vmtk import pypes
from vmtk import vtkvmtk


class vmtkImageFeatureCorrection(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Image = None
        self.LevelSets = None
        self.Sigma = 1.0
        self.ScaleValue = 0.02
        self.NegateLevelSets = False
        self.ComputeScaleValueFromInput = True

        self.SetScriptName('vmtkimagefeaturecorrection')
        self.SetScriptDoc('correct a feature image (e.g. remove influence of bone and/or air from CT-based feature images)')
        self.SetInputMembers([
            ['Image','i','vtkImageData',1,'','the input feature image','vmtkimagereader'],
            ['LevelSets','levelsets','vtkImageData',1,'','the input level sets','vmtkimagereader'],
            ['Sigma','sigma','float',1,'(0.0,)'],
            ['ScaleValue','scale','float',1,'(0.0,)'],
            ['ComputeScaleValueFromInput','scalefrominput','bool',1,''],
            ['NegateLevelSets','negate','bool',1,'']
            ])
        self.SetOutputMembers([
            ['Image','o','vtkImageData',1,'','the output image','vmtkimagewriter']
            ])

    def Execute(self):

        if self.Image == None:
            self.PrintError('Error: No input image.')

        if self.LevelSets == None:
            self.PrintError('Error: No input level sets.')

        if self.NegateLevelSets:
            negateFilter = vtk.vtkImageMathematics()
            negateFilter.SetInputData(self.LevelSets)
            negateFilter.SetOperationToMultiplyByK()
            negateFilter.SetConstantK(-1.0)
            negateFilter.Update()
            self.LevelSets = negateFilter.GetOutput()

        sigmoid = vtkvmtk.vtkvmtkLevelSetSigmoidFilter()
        sigmoid.SetInputData(self.Image)
        sigmoid.SetLevelSetsImage(self.LevelSets)
        sigmoid.SetSigma(self.Sigma)
        sigmoid.SetScaleValue(self.ScaleValue)
        if self.ComputeScaleValueFromInput:
            sigmoid.ComputeScaleValueFromInputOn()
        else:
            sigmoid.ComputeScaleValueFromInputOff()
        sigmoid.Update()

        self.Image = sigmoid.GetOutput()


if __name__=='__main__':

    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkdelaunayvoronoi.py,v $
## Language:  Python
## Date:      $Date: 2006/07/17 09:52:56 $
## Version:   $Revision: 1.20 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
import sys

from vmtk import vtkvmtk
from vmtk import vmtkrenderer
from vmtk import pypes


class vmtkNonManifoldSurfaceChecker(object):

    def __init__(self):

        self.Surface = 0

        self.NumberOfNonManifoldEdges = 0
        self.Report = 0
        self.NonManifoldEdgePointIds = vtk.vtkIdList()

        self.PrintError = None

    def Execute(self):

        if (self.Surface == 0):
            self.PrintError('NonManifoldSurfaceChecker error: Surface not set')
            return

        self.NonManifoldEdgesFound = 0
        self.Report = ''
        self.NonManifoldEdgePointIds.Initialize()

        neighborhoods = vtkvmtk.vtkvmtkNeighborhoods()
        neighborhoods.SetNeighborhoodTypeToPolyDataManifoldNeighborhood()
        neighborhoods.SetDataSet(self.Surface)
        neighborhoods.Build()

        neighborCellIds = vtk.vtkIdList()
        cellPointIds = vtk.vtkIdList()

        self.Surface.BuildCells()
        self.Surface.BuildLinks(0)

        numberOfNonManifoldEdges = 0

        for i in range(neighborhoods.GetNumberOfNeighborhoods()):

            neighborhood = neighborhoods.GetNeighborhood(i)

            for j in range(neighborhood.GetNumberOfPoints()):

                neighborId = neighborhood.GetPointId(j)

                if (i<neighborId):

                    neighborCellIds.Initialize()
                    self.Surface.GetCellEdgeNeighbors(-1,i,neighborId,neighborCellIds)

                    if (neighborCellIds.GetNumberOfIds()>2):

                        numberOfNonManifoldEdges = numberOfNonManifoldEdges + 1

                        self.Report = self.Report +  "Non-manifold edge found" + str(i) + ' ' + str(neighborId) + '.\n'

                        self.NonManifoldEdgePointIds.InsertNextId(i)
                        self.NonManifoldEdgePointIds.InsertNextId(neighborId)


class vmtkDelaunayVoronoi(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Surface = None
        self.FlipNormals = 0
        self.CapDisplacement = 0.0
        self.RadiusArrayName = 'MaximumInscribedSphereRadius'
        self.CheckNonManifold = 0

        self.RemoveSubresolutionTetrahedra = 0
        self.SubresolutionFactor = 1.0

        self.SimplifyVoronoi = 0

        self.UseTetGen = 0
        self.TetGenDetectInter = 1

        self.DelaunayTolerance = 0.001

        self.DelaunayTessellation = None
        self.VoronoiDiagram = None
        self.PoleIds = None

        self.Mesh = None

        self.SetScriptName('vmtkdelaunayvoronoi')
        self.SetScriptDoc('calculate the delaunay tesellation, voronoi diagram, and voronoi poleIds of an input surface')
        self.SetInputMembers([
            ['Surface','i','vtkPolyData',1,'','the input surface','vmtksurfacereader'],
            ['CheckNonManifold','nonmanifoldcheck','bool',1,'','toggle checking the surface for non-manifold edges'],
            ['FlipNormals','flipnormals','bool',1,'','flip normals after outward normal computation; outward oriented normals must be computed for the removal of outer tetrahedra; the algorithm might fail so for weird geometries, so changing this might solve the problem'],
            ['CapDisplacement','capdisplacement','float',1,'','displacement of the center points of caps at open profiles along their normals (avoids the creation of degenerate tetrahedra)'],
            ['DelaunayTolerance','delaunaytolerance','float',1,'','tolerance for evaluating coincident points during Delaunay tessellation, evaluated as a fraction of the bounding box'],
            ['RadiusArrayName','radiusarray','str',1,'','name of the array where radius values of maximal inscribed spheres have to be stored'],
            ['DelaunayTessellation','delaunaytessellation','vtkUnstructuredGrid',1,'','optional input Delaunay tessellation'],
            ['RemoveSubresolutionTetrahedra','removesubresolution','bool',1,'','toggle removal of subresolution tetrahedra from Delaunay tessellation'],
            ['SubresolutionFactor','subresolutionfactor','float',1,'(0.0,)','factor for removal of subresolution tetrahedra, expressing the size of the circumsphere relative to the local edge length size of surface triangles'],
            ['SimplifyVoronoi','simplifyvoronoi','bool',1,'','toggle simplification of Voronoi diagram'],
            ['UseTetGen','usetetgen','bool',1,'','toggle use TetGen to compute Delaunay tessellation'],
            ['TetGenDetectInter','tetgendetectinter','bool',1,'','TetGen option']])
        self.SetOutputMembers([
            ['RadiusArrayName','radiusarray','str',1,'','name of the array where radius values of maximal inscribed spheres are stored'],
            ['DelaunayTessellation','delaunaytessellation','vtkUnstructuredGrid',1,'','','vmtkmeshwriter'],
            ['VoronoiDiagram','voronoidiagram','vtkPolyData',1,'','','vmtksurfacewriter'],
            ['Mesh','omesh','vtkUnstructuredGrid',1,'','conveniently named DelaunayTessellation output','vmtkmeshwriter'],
            ['Surface','osurface','vtkPolyData',1,'','conveniently named VoronoiDiagram output','vmtksurfacewriter'],
            ['PoleIds','poleids','vtkIdList',1]])

    def Execute(self):

        if self.Surface == None:
            self.PrintError('Error: No input surface.')

        if self.CheckNonManifold:
            self.PrintLog('NonManifold check.')
            nonManifoldChecker = vmtkNonManifoldSurfaceChecker()
            nonManifoldChecker.Surface = self.Surface
            nonManifoldChecker.PrintError = self.PrintError
            nonManifoldChecker.Execute()

            if nonManifoldChecker.NumberOfNonManifoldEdges > 0:
                self.PrintLog(nonManifoldChecker.Report)
                return

        self.PrintLog('Cleaning surface.')
        surfaceCleaner = vtk.vtkCleanPolyData()
        surfaceCleaner.SetInputData(self.Surface)
        surfaceCleaner.Update()

        self.PrintLog('Triangulating surface.')
        surfaceTriangulator = vtk.vtkTriangleFilter()
        surfaceTriangulator.SetInputConnection(surfaceCleaner.GetOutputPort())
        surfaceTriangulator.PassLinesOff()
        surfaceTriangulator.PassVertsOff()
        surfaceTriangulator.Update()

        surfaceCapper = vtkvmtk.vtkvmtkCapPolyData()
        surfaceCapper.SetInputConnection(surfaceTriangulator.GetOutputPort())
        surfaceCapper.SetDisplacement(self.CapDisplacement)
        surfaceCapper.SetInPlaneDisplacement(self.CapDisplacement)
        surfaceCapper.Update()

        capCenterIds = surfaceCapper.GetCapCenterIds()

        # Since VTK 9.4, vtkPolyDataNormals passes pre-existing normals through
        # unchanged instead of recomputing them; strip inherited normals so
        # outward-oriented normals are always recomputed for the internal
        # tetrahedra classification.
        cappedSurface = vtk.vtkPolyData()
        cappedSurface.ShallowCopy(surfaceCapper.GetOutput())
        cappedSurface.GetPointData().SetNormals(None)
        cappedSurface.GetCellData().SetNormals(None)

        surfaceNormals = vtk.vtkPolyDataNormals()
        surfaceNormals.SetInputData(cappedSurface)
        surfaceNormals.SplittingOff()
        surfaceNormals.AutoOrientNormalsOn()
        surfaceNormals.SetFlipNormals(self.FlipNormals)
        surfaceNormals.ComputePointNormalsOn()
        surfaceNormals.ConsistencyOn()
        surfaceNormals.Update()

        inputSurface = surfaceNormals.GetOutput()

        if self.UseTetGen:
            self.PrintLog('Running TetGen.')
            from vmtk import vmtkscripts
            surfaceToMesh = vmtkscripts.vmtkSurfaceToMesh()
            surfaceToMesh.Surface = inputSurface
            surfaceToMesh.Execute()
            tetgen = vmtkscripts.vmtkTetGen()
            tetgen.Mesh = surfaceToMesh.Mesh
            tetgen.PLC = 1
            tetgen.NoMerge = 1
            tetgen.Quality = 0
            if self.TetGenDetectInter:
                tetgen.DetectInter = 1
                tetgen.NoMerge = 0
            tetgen.OutputSurfaceElements = 0
            tetgen.Execute()
            self.DelaunayTessellation = tetgen.Mesh
        else:
            delaunayTessellator = vtk.vtkDelaunay3D()
            delaunayTessellator.CreateDefaultLocator()
            delaunayTessellator.SetInputConnection(surfaceNormals.GetOutputPort())
            delaunayTessellator.SetTolerance(self.DelaunayTolerance)
            delaunayTessellator.Update()
            self.DelaunayTessellation = delaunayTessellator.GetOutput()

        normalsArray = surfaceNormals.GetOutput().GetPointData().GetNormals()
        self.DelaunayTessellation.GetPointData().AddArray(normalsArray)

        internalTetrahedraExtractor = vtkvmtk.vtkvmtkInternalTetrahedraExtractor()
        internalTetrahedraExtractor.SetInputData(self.DelaunayTessellation)
        internalTetrahedraExtractor.SetOutwardNormalsArrayName(normalsArray.GetName())
        if self.RemoveSubresolutionTetrahedra:
            internalTetrahedraExtractor.RemoveSubresolutionTetrahedraOn()
            internalTetrahedraExtractor.SetSubresolutionFactor(self.SubresolutionFactor)
            internalTetrahedraExtractor.SetSurface(inputSurface)
        if capCenterIds.GetNumberOfIds() > 0:
          internalTetrahedraExtractor.UseCapsOn()
          internalTetrahedraExtractor.SetCapCenterIds(capCenterIds)
        internalTetrahedraExtractor.Update()

        self.DelaunayTessellation = internalTetrahedraExtractor.GetOutput()

        voronoiDiagramFilter = vtkvmtk.vtkvmtkVoronoiDiagram3D()
        voronoiDiagramFilter.SetInputData(self.DelaunayTessellation)
        voronoiDiagramFilter.SetRadiusArrayName(self.RadiusArrayName)
        voronoiDiagramFilter.Update()

        self.PoleIds = voronoiDiagramFilter.GetPoleIds()

        self.VoronoiDiagram = voronoiDiagramFilter.GetOutput()

        if self.SimplifyVoronoi:
            voronoiDiagramSimplifier = vtkvmtk.vtkvmtkSimplifyVoronoiDiagram()
            voronoiDiagramSimplifier.SetInputConnection(voronoiDiagramFilter.GetOutputPort())
            voronoiDiagramSimplifier.SetUnremovablePointIds(voronoiDiagramFilter.GetPoleIds())
            voronoiDiagramSimplifier.Update()
            self.VoronoiDiagram = voronoiDiagramSimplifier.GetOutput()

        self.Mesh = self.DelaunayTessellation
        self.Surface = self.VoronoiDiagram


if __name__=='__main__':

    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkimagevolumeviewer.py,v $
## Language:  Python
## Date:      $Date: 2018/04/02 12:35:13 $
## Version:   $Revision: 0.1 $

##   Copyright (c) Richard Izzo, Luca Antiga. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
import sys

from vmtk import vtkvmtk
from vmtk import vmtkrenderer
from vmtk import vmtkimageviewer
from vmtk import pypes
import xml.etree.ElementTree as ET
import os


class vmtkImageVolumeViewer(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Image = None
        self.Volume = None
        self.vmtkRenderer = None
        self.OwnRenderer = 0
        self.Display = 1
        self.ArrayName = ''

        self.Preset = 'CT-Coronary-Arteries-2'
        self.VolumeProperty = None
        self.VolumeRenderingMethod = 'default'
        self.BoxOutline = 0

        self.ColorTransferFunction = None
        self.GradientOpacityTransferFunction = None
        self.OpacityTransferFunction = None
        self.InterpolationType = None
        self.Shade = None
        self.SpecularPower = None
        self.Specular = None
        self.Diffuse = None
        self.Ambient = None

        self.SetScriptName('vmtkimagevolumeviewer')
        self.SetScriptDoc('display a 3D image within a volume renderer')
        self.SetInputMembers([
            ['Image','i','vtkImageData',1,'','the input image','vmtkimagereader'],
            ['ArrayName','array','str',1,'','name of the array to display'],
            ['vmtkRenderer','renderer','vmtkRenderer',1,'','external renderer'],
            ['Display','display','bool',1,'','toggle rendering'],
            ['Preset','preset','str',1,'["CT-AAA","CT-AAA2","CT-Bone","CT-Bones",\
                                        "CT-Cardiac","CT-Cardiac2","CT-Cardiac3","CT-Chest-Contrast-Enhanced",\
                                        "CT-Chest-Vessels","CT-Coronary-Arteries","CT-Coronary-Arteries-2",\
                                        "CT-Coronary-Arteries-3","CT-Cropped-Volume-Bone","CT-Fat","CT-Liver-Vasculature",\
                                        "CT-Lung","CT-MIP","CT-Muscle","CT-Pulmonary-Arteries","CT-Soft-Tissue","MR-Angio",\
                                        "MR-Default","MR-MIP","MR-T2-Brain","DTI-FA-Brain"]','sets the color map, opacity, and lighting applied to visualize the rendered volume'],
            ['VolumeRenderingMethod','rendermethod','str',1,'["default","gpu","ospray","raycast"]','toggle rendering method. By default will auto detect hardware capabilities and select best render method. If desired, can set to RayCast, OSPRay, or GPU rendering. Note, may crash if GPU method is set and no compatible GPU is available on the system. Suggested to leave at default value.'],
            ['BoxOutline','box','bool',1,'','display a box-shaped outline around the edges of the volume data']
            ])
        self.SetOutputMembers([
            ['Image','o','vtkImageData',1,'','the output image','vmtkimagewriter'],
            ['ColorTransferFunction','colorfunction','vtkColorTransferFunction',1,'',''],
            ['GradientOpacityTransferFunction','gradientopacityfunction','vtkPiecewiseFunction',1,'',''],
            ['OpacityTransferFunction','opacityfunction','vtkPiecewiseFunction',1,'',''],
            ['InterpolationType','interpolation','int',1,'',''],
            ['Shade','shade','int',1,'',''],
            ['SpecularPower','specularpower','float',1,'',''],
            ['Specular','specular','float',1,'',''],
            ['Diffuse','diffuse','float',1,'',''],
            ['Ambient','ambient','float',1,'','']
            ])

    def BuildVTKPiecewiseFunction(self, pointsList):
        '''Assign values to a newly created vtkPiecewiseFunction

        Arguments:
            Inputs:
                pointsList (list of tuples): a list of tuples, each of size=2. The first value
                    specifies the xValue of the point to add (aka image scalar value). The second
                    value specifies the yValue of the point to add (aka, value at that image scalar)

            Outputs:
                piecewiseFunction (vtkPiecewiseFunction object): containing points at tupples in the points list
        '''

        piecewiseFunction = vtk.vtkPiecewiseFunction()
        for xValue, yValue in pointsList:
            piecewiseFunction.AddPoint(xValue, yValue)
        return piecewiseFunction

    def BuildVTKColorTransferFunction(self, pointsList):
        '''Assign values to a newly created vtkColorTransferFunction

        Arguments:
            Inputs:
                pointsList (list of tuples): a list of tuples, each of size=4. The first value
                    specifies the xValue of the point to add (aka image scalar value). The second,
                    third, and fourth values specify the RGB value to assign at that point

            Outputs:
                colorTransferFunction (vtkColorTransferFunction object): containing RGB points at tupples in the points list
        '''
        colorTransferFunction = vtk.vtkColorTransferFunction()
        for xValue, RVal, GVal, BVal in pointsList:
            colorTransferFunction.AddRGBPoint(xValue, RVal, GVal, BVal)
        return colorTransferFunction

    def SetPresetVolumeProperty(self):
        '''Create the transfer functions and set volume properties in a vtkVolumeProperty object'''

        # parse the xml file which is loaded in the same path as the vmtkimagevolumeviewer.py file at runtime
        presetElementTree = ET.parse(os.path.join(os.path.dirname(__file__), 'share', 'vmtkimagevolumeviewerpresets.xml'))
        presetRoot = presetElementTree.getroot()
        volProperties = presetRoot.findall('VolumeProperty[@name="' + self.Preset + '"]') # should return a list with only one element
        volPropertiesDict = volProperties[0].attrib

        def _chunks(l, n):
            """Yield successive n-sized chunks from l."""
            PY3 = sys.version_info[0] == 3
            if PY3:
                range_func = range
            else:
                range_func = xrange
            for i in range_func(0, len(l), n):
                yield l[i:i + n]

        # need to convert the space seperated string displaying values into a list of floats
        colorList = [float(i) for i in volPropertiesDict['colorTransfer'].split()]
        gradientOpacityList = [float(i) for i in volPropertiesDict['gradientOpacity'].split()]
        opacityList = [float(i) for i in volPropertiesDict['scalarOpacity'].split()]

        # create an array of arrays with each list split into subarrays of desired size
        colorMapList = _chunks(colorList, 4)
        gradientOpacityMapList = _chunks(gradientOpacityList, 2)
        opacityMapList = _chunks(opacityList, 2)

        # create vtk objects from the mapped lists (now arrays of arrays)
        self.ColorTransferFunction = self.BuildVTKColorTransferFunction(colorMapList)
        self.GradientOpacityTransferFunction = self.BuildVTKPiecewiseFunction(gradientOpacityMapList)
        self.OpacityTransferFunction = self.BuildVTKPiecewiseFunction(opacityMapList)

        # assign other properties from the element tree to variables with the appropriate type
        self.InterpolationType = int(volPropertiesDict['interpolation'])
        self.Shade = int(volPropertiesDict['shade'])
        self.SpecularPower = float(volPropertiesDict['specularPower'])
        self.Specular = float(volPropertiesDict['specular'])
        self.Diffuse = float(volPropertiesDict['diffuse'])
        self.Ambient = float(volPropertiesDict['ambient'])

        # set transfer function and lighting properties of the vtk volume object
        volumeProperty = vtk.vtkVolumeProperty()
        volumeProperty.SetScalarOpacity(self.OpacityTransferFunction)
        volumeProperty.SetGradientOpacity(self.GradientOpacityTransferFunction)
        volumeProperty.SetColor(self.ColorTransferFunction)
        volumeProperty.SetInterpolationType(self.InterpolationType)
        volumeProperty.SetShade(self.Shade)
        volumeProperty.SetSpecularPower(self.SpecularPower)
        volumeProperty.SetSpecular(self.Specular)
        volumeProperty.SetDiffuse(self.Diffuse)
        volumeProperty.SetAmbient(self.Ambient)

        self.VolumeProperty = volumeProperty

    def PresetCallback(self, obj):
        '''Set render window to render the next volume property when callback key is pressed'''

        if not self.VolumeProperty:
            return

        if self.Preset == "CT-AAA":
            self.Preset = "CT-AAA2"
        elif self.Preset == "CT-AAA2":
            self.Preset = "CT-Bone"
        elif self.Preset == "CT-Bone":
            self.Preset = "CT-Bones"
        elif self.Preset == "CT-Bones":
            self.Preset = "CT-Cardiac"
        elif self.Preset == "CT-Cardiac":
            self.Preset = "CT-Cardiac2"
        elif self.Preset == "CT-Cardiac2":
            self.Preset = "CT-Cardiac3"
        elif self.Preset == "CT-Cardiac3":
            self.Preset = "CT-Chest-Contrast-Enhanced"
        elif self.Preset == "CT-Chest-Contrast-Enhanced":
            self.Preset = "CT-Chest-Vessels"
        elif self.Preset == "CT-Chest-Vessels":
            self.Preset = "CT-Coronary-Arteries"
        elif self.Preset == "CT-Coronary-Arteries":
            self.Preset = "CT-Coronary-Arteries-2"
        elif self.Preset == "CT-Coronary-Arteries-2":
            self.Preset = "CT-Coronary-Arteries-3"
        elif self.Preset == "CT-Coronary-Arteries-3":
            self.Preset = "CT-Cropped-Volume-Bone"
        elif self.Preset == "CT-Cropped-Volume-Bone":
            self.Preset = "CT-Fat"
        elif self.Preset == "CT-Fat":
            self.Preset = "CT-Liver-Vasculature"
        elif self.Preset == "CT-Liver-Vasculature":
            self.Preset = "CT-Lung"
        elif self.Preset == "CT-Lung":
            self.Preset = "CT-MIP"
        elif self.Preset == "CT-MIP":
            self.Preset = "CT-Muscle"
        elif self.Preset == "CT-Muscle":
            self.Preset = "CT-Pulmonary-Arteries"
        elif self.Preset == "CT-Pulmonary-Arteries":
            self.Preset = "CT-Soft-Tissue"
        elif self.Preset == "CT-Soft-Tissue":
            self.Preset = "MR-Angio"
        elif self.Preset == "MR-Angio":
            self.Preset = "MR-Default"
        elif self.Preset == "MR-Default":
            self.Preset = "MR-MIP"
        elif self.Preset == "MR-MIP":
            self.Preset = "MR-T2-Brain"
        elif self.Preset == "MR-T2-Brain":
            self.Preset = "CT-AAA"

        self.PrintLog("switched Preset to: " + self.Preset, 1)

        self.SetPresetVolumeProperty()
        self.Volume.SetProperty(self.VolumeProperty)
        self.vmtkRenderer.RenderWindow.Render()

    def BuildView(self):

        # ensure python 2-3 compatibility for checking string type (difference with unicode str handeling)
        PY3 = sys.version_info[0] == 3

        if not self.vmtkRenderer:
            self.vmtkRenderer = vmtkrenderer.vmtkRenderer()
            self.vmtkRenderer.Initialize()
            self.OwnRenderer = 1

        self.vmtkRenderer.RegisterScript(self)

        if (self.ArrayName != ''):
            self.Image.GetPointData().SetActiveScalars(self.ArrayName)

        if PY3:
            string_types = str,
        else:
            string_types = basestring,
        if not isinstance(self.VolumeRenderingMethod, string_types):
            self.PrintError('Specified Rendering Method is not of required "string" type')

        # create volume mapper and apply requested volume rendering method
        volumeMapper = vtk.vtkSmartVolumeMapper()
        if self.VolumeRenderingMethod.lower() == 'default':
            volumeMapper.SetRequestedRenderModeToDefault()
        elif self.VolumeRenderingMethod.lower() == 'gpu':
            volumeMapper.SetRequestedRenderModeToGPU()
        elif self.VolumeRenderingMethod.lower() == 'ospray':
            volumeMapper.SetRequestedRenderModeToOSPRay()
        elif self.VolumeRenderingMethod.lower() == 'raycast':
            volumeMapper.SetRequestedRenderModeToRayCast()
        else:
            self.PrintError('Specified Rendering Method: ' + self.VolumeRenderingMethod + ' not supported. Please choose from ["default", "gpu", "ospray", "raycast"]')
        volumeMapper.SetInputData(self.Image)
        volumeMapper.SetBlendModeToComposite()

        # create the volume from the mapper (defining image data and render mode) with the defined properties
        # this is the last pipeline step applied. self.Volume just needs to now be added to the Renderer
        self.SetPresetVolumeProperty()
        self.Volume = vtk.vtkVolume()
        self.Volume.SetMapper(volumeMapper)
        self.Volume.SetProperty(self.VolumeProperty)

        # add the actors to the renderer
        self.vmtkRenderer.Renderer.AddVolume(self.Volume)

        # This next section is responsible for creating the box outline around the volume's data extent.
        if self.BoxOutline:
            outline = vtk.vtkOutlineFilter()
            outline.SetInputData(self.Image)
            outlineMapper = vtk.vtkPolyDataMapper()
            outlineMapper.SetInputConnection(outline.GetOutputPort())
            outlineActor = vtk.vtkActor()
            outlineActor.SetMapper(outlineMapper)

            # The SetInteractor method is how 3D widgets are associated with the
            # render window interactor. Internally, SetInteractor sets up a bunch
            # of callbacks using the Command/Observer mechanism (AddObserver()).
            boxWidget = vtk.vtkBoxWidget()
            boxWidget.SetInteractor(self.vmtkRenderer.RenderWindowInteractor)
            boxWidget.SetPlaceFactor(1.0)

            # Place the interactor initially. The output of the reader is used to
            # place the box widget.
            boxWidget.SetInputData(self.Image)
            boxWidget.PlaceWidget()
            boxWidget.InsideOutOn()

            # Add the actors to the renderer
            self.vmtkRenderer.Renderer.AddActor(outlineActor)

        # add preset callback
        self.vmtkRenderer.AddKeyBinding('w','Change volume render property.',self.PresetCallback)

        if (self.Display == 1):
            self.vmtkRenderer.Render()

        if self.OwnRenderer:
            self.vmtkRenderer.Deallocate()

    def Execute(self):

        if (self.Image == None) & (self.Display == 1):
            self.PrintError('Error: no Image.')

        self.BuildView()



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkimagesmoothing.py,v $
## Language:  Python
## Date:      $Date: 2006/07/17 09:53:14 $
## Version:   $Revision: 1.8 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
import sys

from vmtk import pypes
from vmtk import vtkvmtk


class vmtkImageSmoothing(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Image = None

        self.Method = 'gauss'
        self.StandardDeviation = 1.0
        self.RadiusFactor = 5.0
        self.Dimensionality = 3
        self.EnhancedImage = None

        # This is the default value from Slicer 3 which assums pixel spacing = 1.0: (PixelSpacing)/2^{N+1}->1.0/2^{3+1}
        # If AutoCalculateTimeStep is enabled, this will be overwritten with the value calculated with the actual image
        # pixel spacing and number of dimensions.
        self.TimeStep = 0.0625
        self.AutoCalculateTimeStep = 1
        self.Conductance = 1.0
        self.NumberOfIterations = 5

        self.SetScriptName('vmtkimagesmoothing')
        self.SetScriptDoc('smooth an image with a Gaussian kernel or anisotropic diffusion')
        self.SetInputMembers([
            ['Image','i','vtkImageData',1,'','the input image','vmtkimagereader'],
            ['Method','method','str',1,'["gauss","anisotropic"]'],
            ['StandardDeviation','sigma','float',1,'(0.0,)','the standard deviation of the Gaussian in real space units (gauss)'],
            ['RadiusFactor','radiusfactor','float',1,'(0.0,)','the factor specifing the width of the discrete Gaussian kernel (gauss)'],
            ['Dimensionality','dimensionality','int',1,'(2,3)','the dimensionality of the Aconvolution (gauss)'],
            ['Conductance','conductance','float',1,'(0.0,)','anisotropic diffustion coefficients (anisotropic)'],
            ['NumberOfIterations','iterations','int',1,'(5,)',' number of anisotropic diffusion iterationsl (anisotropic)'],
            ['TimeStep','timestep','float',1,'(1.0E-16,1.0e16)','time step of anisotropic diffusion (anisotropic)'],
            ['AutoCalculateTimeStep','autocalculatetimestep','bool',1,'','auto calculate minimum time step (anisotropic)']
            ])
        self.SetOutputMembers([
            ['Image','o','vtkImageData',1,'','the output image','vmtkimagewriter'],
            ['TimeStep','timestep','float',1],
            ])

    def ApplyGaussianFilter(self):

        spacing = self.Image.GetSpacing()
        pixelStandardDeviations = [self.StandardDeviation/spacing[0],self.StandardDeviation/spacing[1],self.StandardDeviation/spacing[2]]

        smoothingFilter = vtk.vtkImageGaussianSmooth()
        smoothingFilter.SetInputData(self.Image)
        smoothingFilter.SetStandardDeviations(pixelStandardDeviations)
        smoothingFilter.SetRadiusFactor(self.RadiusFactor)
        smoothingFilter.SetDimensionality(self.Dimensionality)
        smoothingFilter.Update()

        self.EnhancedImage = smoothingFilter.GetOutput()

    def ApplyAnisotropicDiffusion(self):

        if self.AutoCalculateTimeStep:
            # equation to calculate the timestep is (PixelSpacing)/2^{N+1}
            minspacing = min(self.Image.GetSpacing())
            #multiply by .995 to make sure its under stability threshold
            self.TimeStep = (minspacing / (2.0 ** (self.Image.GetDataDimension() + 1))) * 0.9995

        grad = vtkvmtk.vtkvmtkAnisotropicDiffusionImageFilter()
        grad.SetInputData(self.Image)
        grad.SetNumberOfIterations(self.NumberOfIterations)
        grad.SetTimeStep(self.TimeStep)
        grad.SetConductance(self.Conductance)
        grad.Update()
        self.EnhancedImage = grad.GetOutput()

    def Execute(self):

        if self.Image == None:
            self.PrintError('Error: No input image.')

        # need to ensure that self.Method variable is a "string"
        # in order to ensure unicode compatibility in python 2 and 3, we check what python version is running,
        # and assign 'str' type if Py3, and 'basestring' type if Py2
        try:
            basestring
        except NameError:
            basestring = str

        if not isinstance(self.Method, basestring):
            self.PrintError('Method Selected (', self.Method, ') is not string type')

        if self.Method.lower() == 'gauss':
            self.ApplyGaussianFilter()
        elif self.Method.lower() == 'anisotropic':
            self.ApplyAnisotropicDiffusion()
        else:
            self.PrintError('Error: unsupported vessel enhancement method (', self.Method, ').')

        self.Image = self.EnhancedImage


if __name__=='__main__':

    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkcenterlineimage.py,v $
## Language:  Python
## Date:      $Date: 2018/04/18 09:52:56 $
## Version:   $Revision: 1.40 $

##   Copyright (c) Richard Izzo, Luca Antiga. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## This algorithm is based off the paper
##     Flux driven medial curve extraction
##     Mellado X., Larrabide I., Hernandez M., Frangi A.
##     Pompeu Fabra University, University of Zaragoza
##     http://hdl.handle.net/1926/560


from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
from vtk.numpy_interface import dataset_adapter as dsa
import numpy as np
import math
import sys

from vmtk import vtkvmtk
from vmtk import vmtkrenderer
from vmtk import vmtksurfacetobinaryimage, vmtksurfacecapper
from vmtk import pypes


class vmtkCenterlineImage(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Surface = None
        self.Image = None

        self.Sigma = 0.5
        self.Threshold = 0.0
        self.PolyDataToImageDataSpacing = [0.3, 0.3, 0.3]

        self.SetScriptName('vmtkcenterlineimage')
        self.SetScriptDoc('Automatically extract a binary skeleton centerline image from a surface using a flux-driven medial curve extraction algorithm. Note: results will be slightly different on different operating systems.')
        self.SetInputMembers([
            ['Surface','i','vtkPolyData',1,'','the input surface','vmtksurfacereader'],
            ['Sigma','sigma','float',1,'(0.0,)','the kernal width of the gaussian used for image smoothing'],
            ['Threshold','threshold','float',1,'(0.0,1.0)','the threshold to filter the output skeleton image at'],
            ['PolyDataToImageDataSpacing','spacing','float',3,'(0.0,)','the sample spacing for the polydata to image data conversion']
            ])
        self.SetOutputMembers([
            ['Image','o','vtkImageData',1,'','the output centerline image','vmtkimagewriter']
            ])

    def Execute(self):
        if self.Surface == None:
            self.PrintError('Error: No Input Surface.')

        # Step 0: Check if the surface has any unfilled holes in it. if it does, cap them.
        fedges = vtk.vtkFeatureEdges()
        fedges.BoundaryEdgesOn()
        fedges.FeatureEdgesOff()
        fedges.ManifoldEdgesOff()
        fedges.SetInputData(self.Surface)
        fedges.Update()

        ofedges = fedges.GetOutput()
        # if the following is not == 0, then the surface contains unfilled holes.
        numEdges = ofedges.GetNumberOfPoints()
        if numEdges >= 1:
            self.PrintLog('Capping unclosed holes in surface.')
            tempcapper = vmtksurfacecapper.vmtkSurfaceCapper()
            tempcapper.Surface = self.Surface
            tempcapper.LogOn = 0
            tempcapper.Interactive = 0
            tempcapper.Execute()
            self.Surface = tempcapper.Surface

        # Step 1: Convert the input surface into an image mask of unsigned char type and spacing = PolyDataToImageDataSpacing
        #         Where voxels lying inside the surface are set to 255 and voxels outside the image are set to value 0.

        # since we are creating a new image container from nothing, calculate the origin, extent, and dimensions for the
        # vtkImageDataObject from the surface parameters.

        self.PrintLog('Converting Surface to Image Mask')
        binaryImageFilter = vmtksurfacetobinaryimage.vmtkSurfaceToBinaryImage()
        binaryImageFilter.Surface = self.Surface
        binaryImageFilter.InsideValue = 255
        binaryImageFilter.LogOn = 0
        binaryImageFilter.OutsideValue = 0
        binaryImageFilter.PolyDataToImageDataSpacing = self.PolyDataToImageDataSpacing
        binaryImageFilter.Execute()

        # Step 2: Feed into the vtkvmtkMedialCurveFilter
        #         This takes the binary image and computes the average outward flux of the image. This is then
        #         used to compute the skeleton image. It returns a binary image where values of 1 are skeleton points
        #         and values of 0 are outside the skeleton. The execution speed of this algorithm is fairly sensetive to
        #         the extent of the input image.
        self.PrintLog('Extracting Centerline Skeleton from Image Mask...')
        medialCurveFilter = vtkvmtk.vtkvmtkMedialCurveFilter()
        medialCurveFilter.SetInputData(binaryImageFilter.Image)
        medialCurveFilter.SetThreshold(self.Threshold)
        medialCurveFilter.SetSigma(self.Sigma)
        medialCurveFilter.Update()

        self.Image = medialCurveFilter.GetOutput()


if __name__=='__main__':
    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtksurfacenormals.py,v $
## Language:  Python
## Date:      $Date: 2005/09/14 09:49:59 $
## Version:   $Revision: 1.7 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
from vmtk import vtkvmtk
import sys

from vmtk import pypes


class vmtkSurfaceNormals(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Surface = None
        self.NormalsArrayName = 'Normals'

        self.ComputeCellNormals = 0
        self.AutoOrientNormals = 1
        self.FlipNormals = 0
        self.Consistency = 1

        self.SetScriptName('vmtksurfacenormals')
        self.SetScriptDoc('compute normals to a surface')
        self.SetInputMembers([
            ['Surface','i','vtkPolyData',1,'','the input surface','vmtksurfacereader'],
            ['NormalsArrayName','normalsarray','str',1,'','name of the array where normals have to be stored'],
            ['AutoOrientNormals','autoorientnormals','bool',1,'','try to auto orient normals outwards'],
            ['Consistency','consistency','bool',1,'','try to orient normals so that neighboring points have similar orientations'],
            ['ComputeCellNormals','cellnormals','bool',1,'','compute cell normals instead of point normals'],
            ['FlipNormals','flipnormals','bool',1,'','flip normals after computing them']
            ])
        self.SetOutputMembers([
            ['Surface','o','vtkPolyData',1,'','the output surface','vmtksurfacewriter']
            ])

    def Execute(self):

        if self.Surface == None:
            self.PrintError('Error: No Surface.')

        # Since the VTK 9.4 refactor, vtkPolyDataNormals passes pre-existing
        # normals through unchanged instead of recomputing them, so the
        # Consistency/AutoOrientNormals/FlipNormals options would silently not
        # be applied when the input surface already carries normals. Drop
        # inherited normals so they are always recomputed.
        surface = vtk.vtkPolyData()
        surface.ShallowCopy(self.Surface)
        surface.GetPointData().SetNormals(None)
        surface.GetCellData().SetNormals(None)

        normalsFilter = vtk.vtkPolyDataNormals()
        normalsFilter.SetInputData(surface)
        normalsFilter.SetAutoOrientNormals(self.AutoOrientNormals)
        normalsFilter.SetFlipNormals(self.FlipNormals)
        normalsFilter.SetConsistency(self.Consistency)
        normalsFilter.SetComputeCellNormals(self.ComputeCellNormals)
        normalsFilter.SplittingOff()
        normalsFilter.Update()

        self.Surface = normalsFilter.GetOutput()

        if self.NormalsArrayName != '':
            self.Surface.GetPointData().GetNormals().SetName(self.NormalsArrayName)


if __name__=='__main__':
    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkmeshaddexternallayer.py,v $
## Language:  Python
## Date:      $$
## Version:   $$

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## Note: this class was contributed by
##       Tangui Morvan
##       Kalkulo AS
##       Simula Research Laboratory

## This class builds an external layer around a mesh.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
from vmtk import vtkvmtk
import sys
from vmtk import vmtkscripts

from vmtk import pypes


class vmtkMeshAddExternalLayer(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Mesh = None

        self.CellEntityIdsArrayName = 'CellEntityIds'

        #Id of the first wall (mesh surface)
        self.SurfaceCellEntityId = 1
        #Id of the first openprofile in the walls
        self.InletOutletCellEntityId = 2
        #Id of the surface cells to extrude
        self.ExtrudeCellEntityId = 1

        self.ThicknessArrayName = ''

        self.Thickness = 1.0
        self.ThicknessRatio = 0.1
        self.MaximumThickness = 1E10

        self.NumberOfSubLayers = 1
        self.SubLayerRatio = 1.0

        self.ConstantThickness = 1;
        self.IncludeSurfaceCells = 0
        self.NegateWarpVectors = 0

        self.IncludeExtrudedOpenProfilesCells = 1
        self.IncludeExtrudedSurfaceCells = 1
        self.IncludeOriginalSurfaceCells = 0

        self.SetScriptName('vmtkmeshaddexternallayer')
        self.SetScriptDoc('create an external prismatic layer from the wall of a mesh and the normals on the wall.')
        self.SetInputMembers([
            ['Mesh','i','vtkUnstructuredGrid',1,'','the input mesh','vmtkmeshreader'],
            ['CellEntityIdsArrayName','entityidsarray','str',1,''],
            ['SurfaceCellEntityId','surfacecellentityid','int',1,'','id of the first surface cells in the entityids list'],
            ['InletOutletCellEntityId','inletoutletcellentityid','int',1],
            ['ExtrudeCellEntityId','extrudecellentityid','int',1,'','id of the surface cells to extrude'],
            ['ThicknessArrayName','thicknessarray','str',1,'','name of the array where scalars defining boundary layer thickness are stored'],
            ['Thickness','thickness','float',1,'','value of constant boundary layer thickness'],
            ['ThicknessRatio','thicknessratio','float',1,'(0.0,)','multiplying factor for boundary layer thickness'],
            ['MaximumThickness','maximumthickness','float',1,'','maximum allowed value for boundary layer thickness'],
            ['NumberOfSubLayers','numberofsublayers','int',1,'(0,)','number of sublayers which the boundary layer has to be made of'],
            ['SubLayerRatio','sublayerratio','float',1,'(0.0,)','ratio between the thickness of two successive boundary layers'],
            ['ConstantThickness','constantthickness','bool',1,'','toggle constant boundary layer thickness'],
            ['IncludeSurfaceCells','includesurfacecells','bool',0,'','include all surface cells in output mesh'],
            ['IncludeExtrudedOpenProfilesCells','includeextrudedopenprofilescells','bool',1,'','include the cells from the open profiles extruded surface in output mesh'],
            ['IncludeExtrudedSurfaceCells','includeextrudedsurfacecells','bool',1,'','include the cells from the extruded surface in output mesh'],
            ['IncludeOriginalSurfaceCells','includeoriginalsurfacecells','bool',1,'','include the cells from the original surfacein output mesh']
            ])
        self.SetOutputMembers([
            ['Mesh','o','vtkUnstructuredGrid',1,'','the output mesh','vmtkmeshwriter'],
            ])

    def Execute(self):

        if self.Mesh == None:
            self.PrintError('Error: No input mesh.')

        if not self.CellEntityIdsArrayName:
            self.PrintError('Error: No input CellEntityIdsArrayName.')
            return

        cellEntityIdsArray = self.Mesh.GetCellData().GetArray(self.CellEntityIdsArrayName)

        #cut off the volumetric elements
        wallThreshold = vtk.vtkThreshold()
        wallThreshold.SetInputData(self.Mesh)
        wallThreshold.ThresholdByUpper(self.SurfaceCellEntityId-0.5)
        wallThreshold.SetInputArrayToProcess(0,0,0,1,self.CellEntityIdsArrayName)
        wallThreshold.Update()

        meshToSurface = vmtkscripts.vmtkMeshToSurface()
        meshToSurface.Mesh = wallThreshold.GetOutput()
        meshToSurface.Execute()

        #Compute the normals for this surface, orientation should be right because the surface is closed
        #TODO: Add option for cell normals in vmtksurfacenormals
        normalsFilter = vtk.vtkPolyDataNormals()
        normalsFilter.SetInputData(meshToSurface.Surface)
        normalsFilter.SetAutoOrientNormals(1)
        normalsFilter.SetFlipNormals(0)
        normalsFilter.SetConsistency(1)
        normalsFilter.SplittingOff()
        normalsFilter.ComputePointNormalsOff()
        normalsFilter.ComputeCellNormalsOn()
        normalsFilter.Update()

        surfaceToMesh = vmtkscripts.vmtkSurfaceToMesh()
        surfaceToMesh.Surface = normalsFilter.GetOutput()
        surfaceToMesh.Execute()

        #Save the current normals
        wallWithBoundariesMesh = surfaceToMesh.Mesh
        savedNormals = vtk.vtkDoubleArray()
        savedNormals.DeepCopy(wallWithBoundariesMesh.GetCellData().GetNormals())
        savedNormals.SetName('SavedNormals')
        wallWithBoundariesMesh.GetCellData().AddArray(savedNormals)

        #cut off the boundaries and other surfaces
        extrudeThresholdLower = vtk.vtkThreshold()
        extrudeThresholdLower.SetInputData(wallWithBoundariesMesh)
        extrudeThresholdLower.ThresholdByLower(self.ExtrudeCellEntityId+0.5)
        extrudeThresholdLower.SetInputArrayToProcess(0,0,0,1,self.CellEntityIdsArrayName)
        extrudeThresholdLower.Update()

        extrudeThresholdUpper = vtk.vtkThreshold()
        extrudeThresholdUpper.SetInputConnection(extrudeThresholdLower.GetOutputPort())
        extrudeThresholdUpper.ThresholdByUpper(self.ExtrudeCellEntityId-0.5)
        extrudeThresholdUpper.SetInputArrayToProcess(0,0,0,1,self.CellEntityIdsArrayName)
        extrudeThresholdUpper.Update()

        meshToSurface = vmtkscripts.vmtkMeshToSurface()
        meshToSurface.Mesh = extrudeThresholdUpper.GetOutput()
        meshToSurface.Execute()

        #Compute cell normals without boundaries
        normalsFilter = vtk.vtkPolyDataNormals()
        normalsFilter.SetInputData(meshToSurface.Surface)
        normalsFilter.SetAutoOrientNormals(1)
        normalsFilter.SetFlipNormals(0)
        normalsFilter.SetConsistency(1)
        normalsFilter.SplittingOff()
        normalsFilter.ComputePointNormalsOn()
        normalsFilter.ComputeCellNormalsOn()
        normalsFilter.Update()

        wallWithoutBoundariesSurface = normalsFilter.GetOutput()

        normals = wallWithoutBoundariesSurface.GetCellData().GetNormals()
        savedNormals = wallWithoutBoundariesSurface.GetCellData().GetArray('SavedNormals')

        math = vtk.vtkMath()

        #If the normal are inverted, recompute the normals with flipping on
        if normals.GetNumberOfTuples() > 0 and math.Dot(normals.GetTuple3(0),savedNormals.GetTuple3(0)) < 0:
            normalsFilter = vtk.vtkPolyDataNormals()
            normalsFilter.SetInputData(meshToSurface.Surface)
            normalsFilter.SetAutoOrientNormals(1)
            normalsFilter.SetFlipNormals(1)
            normalsFilter.SetConsistency(1)
            normalsFilter.SplittingOff()
            normalsFilter.ComputePointNormalsOn()
            normalsFilter.ComputeCellNormalsOn()
            normalsFilter.Update()
            wallWithoutBoundariesSurface = normalsFilter.GetOutput()

        wallWithoutBoundariesSurface.GetPointData().GetNormals().SetName('Normals')

        wallWithoutBoundariesSurface.GetCellData().RemoveArray('SavedNormals')

        surfaceToMesh = vmtkscripts.vmtkSurfaceToMesh()
        surfaceToMesh.Surface = wallWithoutBoundariesSurface
        surfaceToMesh.Execute()

        #Offset to apply to the array
        wallOffset = 0
        if self.IncludeSurfaceCells or self.IncludeOriginalSurfaceCells:
            wallOffset += 1
        if self.IncludeSurfaceCells or self.IncludeExtrudedSurfaceCells:
            wallOffset+=1

        boundaryLayer = vmtkscripts.vmtkBoundaryLayer2()
        boundaryLayer.Mesh = surfaceToMesh.Mesh
        boundaryLayer.WarpVectorsArrayName = 'Normals'
        boundaryLayer.NegateWarpVectors = False
        boundaryLayer.ThicknessArrayName = self.ThicknessArrayName
        boundaryLayer.ConstantThickness = self.ConstantThickness
        boundaryLayer.IncludeSurfaceCells = self.IncludeSurfaceCells
        boundaryLayer.NumberOfSubLayers = self.NumberOfSubLayers
        boundaryLayer.SubLayerRatio = self.SubLayerRatio
        boundaryLayer.Thickness = self.Thickness
        boundaryLayer.ThicknessRatio = self.Thickness
        boundaryLayer.MaximumThickness = self.MaximumThickness
        boundaryLayer.CellEntityIdsArrayName = self.CellEntityIdsArrayName
        boundaryLayer.IncludeExtrudedOpenProfilesCells = self.IncludeExtrudedOpenProfilesCells
        boundaryLayer.IncludeExtrudedSurfaceCells = self.IncludeExtrudedSurfaceCells
        boundaryLayer.IncludeOriginalSurfaceCells = self.IncludeOriginalSurfaceCells
        boundaryLayer.LayerEntityId = self.SurfaceCellEntityId
        boundaryLayer.SurfaceEntityId = self.InletOutletCellEntityId + 1
        if cellEntityIdsArray != None:
            #Append the new surface ids
            idRange = cellEntityIdsArray.GetRange()
            boundaryLayer.OpenProfilesEntityId = idRange[1] + wallOffset + 2
        boundaryLayer.Execute()

        if cellEntityIdsArray != None:
            #offset the previous cellentityids to make room for the new ones
            arrayCalculator = vtk.vtkArrayCalculator()
            arrayCalculator.SetInputData(self.Mesh)
            if vtk.vtkVersion.GetVTKMajorVersion()>=9 or (vtk.vtkVersion.GetVTKMajorVersion()>=8 and vtk.vtkVersion.GetVTKMinorVersion()>=1):
                arrayCalculator.SetAttributeTypeToCellData()
            else:
                arrayCalculator.SetAttributeModeToUseCellData()
            arrayCalculator.AddScalarVariable("entityid",self.CellEntityIdsArrayName,0)
            arrayCalculator.SetFunction("if( entityid > " + str(self.InletOutletCellEntityId-1) +", entityid + " + str(wallOffset) + ", entityid)")
            arrayCalculator.SetResultArrayName('CalculatorResult')
            arrayCalculator.Update()

            #This need to be copied in order to be of the right type (int)
            cellEntityIdsArray.DeepCopy(arrayCalculator.GetOutput().GetCellData().GetArray('CalculatorResult'))

            arrayCalculator.SetFunction("if( entityid > " + str(self.SurfaceCellEntityId-1) +", entityid + 1, entityid)")
            arrayCalculator.Update()

            ##This need to be copied in order to be of the right type (int)
            cellEntityIdsArray.DeepCopy(arrayCalculator.GetOutput().GetCellData().GetArray('CalculatorResult'))

        appendFilter = vtkvmtk.vtkvmtkAppendFilter()
        appendFilter.AddInput(self.Mesh)
        appendFilter.AddInput(boundaryLayer.Mesh)
        appendFilter.Update()

        self.Mesh = appendFilter.GetOutput()


if __name__=='__main__':

    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtksurfaceremeshing.py,v $
## Language:  Python
## Date:      $Date: 2006/02/23 09:27:52 $
## Version:   $Revision: 1.7 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
from vmtk import vtkvmtk
import sys

from vmtk import pypes


class vmtkSurfaceRemeshing(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Surface = None

        self.TargetArea = 1.0
        self.TargetEdgeLength = 1.0
        self.TargetAreaFactor = 1.0
        self.TargetEdgeLengthFactor = 1.0
        self.TriangleSplitFactor = 5.0
        self.MaxArea = 1E16
        self.MinArea = 0.0
        self.MaxEdgeLength = 1E16
        self.MinEdgeLength = 0.0
        self.NumberOfIterations = 10
        self.NumberOfConnectivityOptimizationIterations = 20
        self.CellEntityIdsArrayName = None
        self.TargetAreaArrayName = 'TargetArea'
        self.TargetEdgeLengthArrayName = ''
        self.ElementSizeMode = 'area'
        self.MinAreaFactor = 0.5
        self.AspectRatioThreshold = 1.2
        self.InternalAngleTolerance = 0.0
        self.NormalAngleTolerance = 0.2
        self.CollapseAngleThreshold = 0.2
        self.Relaxation = 0.5
        self.PreserveBoundaryEdges = 0
        self.ExcludeEntityIds = []

        self.SetScriptName('vmtksurfaceremeshing')
        self.SetScriptDoc('remesh a surface using quality triangles')
        self.SetInputMembers([
            ['Surface','i','vtkPolyData',1,'','the input surface','vmtksurfacereader'],
            ['ElementSizeMode','elementsizemode','str',1,'["area","edgelength","areaarray","edgelengtharray"]'],
            ['TargetArea','area','float',1,'(0.0,)'],
            ['TargetAreaArrayName','areaarray','str',1],
            ['TargetEdgeLength','edgelength','float',1,'(0.0,)'],
            ['TargetEdgeLengthArrayName','edgelengtharray','str',1],
            ['TargetAreaFactor','areafactor','float',1,'(0.0,)'],
            ['TargetEdgeLengthFactor','edgelengthfactor','float',1,'(0.0,)'],
            ['TriangleSplitFactor','trianglesplitfactor','float',1,'(0.0,)'],
            ['MaxArea','maxarea','float',1,'(0.0,)'],
            ['MinArea','minarea','float',1,'(0.0,)'],
            ['NumberOfIterations','iterations','int',1,'(0,)'],
            ['NumberOfConnectivityOptimizationIterations','connectivityiterations','int',1,'(0,)'],
            ['CellEntityIdsArrayName','entityidsarray','str',1],
            ['AspectRatioThreshold','aspectratio','float',1,'(0.0,)'],
            ['MinAreaFactor','minareafactor','float',1,'(0.0,)'],
            ['InternalAngleTolerance','internalangletolerance','float',1,'(0.0,)'],
            ['NormalAngleTolerance','normalangletolerance','float',1,'(0.0,)'],
            ['CollapseAngleThreshold','collapseangle','float',1,'(0.0,)'],
            ['Relaxation','relaxation','float',1,'(0.5,)'],
            ['ExcludeEntityIds','exclude','int',-1,''],
            ['PreserveBoundaryEdges','preserveboundary','bool',1]
            ])
        self.SetOutputMembers([
            ['Surface','o','vtkPolyData',1,'','the output surface','vmtksurfacewriter']
            ])

    def Execute(self):

        if self.Surface == None:
            self.PrintError('Error: No input surface.')

        cleaner = vtk.vtkCleanPolyData()
        cleaner.SetInputData(self.Surface)
        cleaner.Update()

        triangleFilter = vtk.vtkTriangleFilter()
        triangleFilter.SetInputConnection(cleaner.GetOutputPort())
        triangleFilter.Update()

        self.Surface = triangleFilter.GetOutput()

        if self.ElementSizeMode == 'edgelength':
            self.TargetArea = 0.25 * 3.0**0.5 * self.TargetEdgeLength**2
        elif self.ElementSizeMode == 'edgelengtharray':
            calculator = vtk.vtkArrayCalculator()
            calculator.SetInputData(self.Surface)
            calculator.AddScalarArrayName(self.TargetEdgeLengthArrayName,0)
            calculator.SetFunction("%f^2 * 0.25 * sqrt(3) * %s^2" % (self.TargetEdgeLengthFactor,self.TargetEdgeLengthArrayName))
            calculator.SetResultArrayName(self.TargetAreaArrayName)
            calculator.Update()
            self.MaxArea = 0.25 * 3.0**0.5 * self.MaxEdgeLength**2
            self.MinArea = 0.25 * 3.0**0.5 * self.MinEdgeLength**2
            self.Surface = calculator.GetOutput()

        excludedIds = vtk.vtkIdList()
        if self.ExcludeEntityIds:
            for excludedId in self.ExcludeEntityIds:
                excludedIds.InsertNextId(excludedId)

        surfaceRemeshing = vtkvmtk.vtkvmtkPolyDataSurfaceRemeshing()
        surfaceRemeshing.SetInputData(self.Surface)
        if self.CellEntityIdsArrayName:
            surfaceRemeshing.SetCellEntityIdsArrayName(self.CellEntityIdsArrayName)
        if self.ElementSizeMode in ['area','edgelength']:
            surfaceRemeshing.SetElementSizeModeToTargetArea()
        elif self.ElementSizeMode in ['areaarray','edgelengtharray']:
            surfaceRemeshing.SetElementSizeModeToTargetAreaArray()
            surfaceRemeshing.SetTargetAreaArrayName(self.TargetAreaArrayName)
        else:
            self.PrintError('Error: unsupported ElementSizeMode.')
        surfaceRemeshing.SetTargetArea(self.TargetArea)
        surfaceRemeshing.SetTargetAreaFactor(self.TargetAreaFactor)
        surfaceRemeshing.SetTriangleSplitFactor(self.TriangleSplitFactor)
        surfaceRemeshing.SetMaxArea(self.MaxArea)
        surfaceRemeshing.SetMinArea(self.MinArea)
        surfaceRemeshing.SetNumberOfIterations(self.NumberOfIterations)
        surfaceRemeshing.SetNumberOfConnectivityOptimizationIterations(self.NumberOfConnectivityOptimizationIterations)
        surfaceRemeshing.SetRelaxation(self.Relaxation)
        surfaceRemeshing.SetMinAreaFactor(self.MinAreaFactor)
        surfaceRemeshing.SetAspectRatioThreshold(self.AspectRatioThreshold)
        surfaceRemeshing.SetInternalAngleTolerance(self.InternalAngleTolerance)
        surfaceRemeshing.SetNormalAngleTolerance(self.NormalAngleTolerance)
        surfaceRemeshing.SetCollapseAngleThreshold(self.CollapseAngleThreshold)
        surfaceRemeshing.SetPreserveBoundaryEdges(self.PreserveBoundaryEdges)
        surfaceRemeshing.SetExcludedEntityIds(excludedIds)
        surfaceRemeshing.Update()

        self.Surface = surfaceRemeshing.GetOutput()


if __name__=='__main__':

    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtksurfacecapper.py,v $
## Language:  Python
## Date:      $Date: 2006/07/17 09:53:14 $
## Version:   $Revision: 1.8 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import sys
import vtk

from vmtk import vtkvmtk
from vmtk import pypes


class vmtkSurfaceCapper(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Surface = None
        self.TriangleOutput = 1
        self.CellEntityIdsArrayName = 'CellEntityIds'
        self.CellEntityIdOffset = 1
        self.Interactive = 1
        self.Method = 'simple'
        self.ConstraintFactor = 1.0
        self.NumberOfRings = 8

        self.vmtkRenderer = None
        self.OwnRenderer = 0

        self.SetScriptName('vmtksurfacecapper')
        self.SetScriptDoc('add caps to the holes of a surface, assigning an id to each cap for easy specification of boundary conditions.')
        self.SetInputMembers([
            ['Surface','i','vtkPolyData',1,'','the input surface','vmtksurfacereader'],
            ['Method','method','str',1,'["simple","centerpoint","smooth","annular","concaveannular"]','capping method'],
            ['TriangleOutput','triangle','bool',1,'','toggle triangulation of the output'],
            ['CellEntityIdsArrayName','entityidsarray','str',1,'','name of the array where the id of the caps have to be stored'],
            ['CellEntityIdOffset','entityidoffset','int',1,'(0,)','offset for entity ids'],
            ['ConstraintFactor','constraint','float',1,'','amount of influence of the shape of the surface near the boundary on the shape of the cap ("smooth" method only)'],
            ['NumberOfRings','rings','int',1,'(0,)','number of rings composing the cap ("smooth" method only)'],
            ['Interactive','interactive','bool',1],
            ['vmtkRenderer','renderer','vmtkRenderer',1,'','external renderer']
            ])
        self.SetOutputMembers([
            ['Surface','o','vtkPolyData',1,'','the output surface','vmtksurfacewriter'],
            ['CellEntityIdsArrayName','entityidsarray','str',1,'','name of the array where the id of the caps are stored']
            ])

    def LabelValidator(self,text):
        import string
        if not text:
            return 0
        if not text.split():
            return 0
        for char in text:
            if char not in string.digits + " ":
                return 0
        return 1

    def Execute(self):

        if self.Surface == None:
            self.PrintError('Error: No input surface.')

#        cleaner = vtk.vtkCleanPolyData()
#        cleaner.SetInput(self.Surface)
#        cleaner.Update()
#
#        triangleFilter = vtk.vtkTriangleFilter()
#        triangleFilter.SetInput(cleaner.GetOutput())
#        triangleFilter.Update()
#
#        self.Surface = triangleFilter.GetOutput()

        boundaryIds = vtk.vtkIdList()

        if self.Interactive:
            if not self.vmtkRenderer:
                from vmtk import vmtkrenderer
                self.vmtkRenderer = vmtkrenderer.vmtkRenderer()
                self.vmtkRenderer.Initialize()
                self.OwnRenderer = 1

            self.vmtkRenderer.RegisterScript(self)

            boundaryExtractor = vtkvmtk.vtkvmtkPolyDataBoundaryExtractor()
            boundaryExtractor.SetInputData(self.Surface)
            boundaryExtractor.Update()

            boundaries = boundaryExtractor.GetOutput()
            numberOfBoundaries = boundaries.GetNumberOfCells()
            seedPoints = vtk.vtkPoints()
            for i in range(numberOfBoundaries):
                barycenter = [0.0, 0.0, 0.0]
                vtkvmtk.vtkvmtkBoundaryReferenceSystems.ComputeBoundaryBarycenter(boundaries.GetCell(i).GetPoints(),barycenter)
                seedPoints.InsertNextPoint(barycenter)
            seedPolyData = vtk.vtkPolyData()
            seedPolyData.SetPoints(seedPoints)
            labelsMapper = vtk.vtkLabeledDataMapper();
            labelsMapper.SetInputData(seedPolyData)
            labelsMapper.SetLabelModeToLabelIds()
            labelsActor = vtk.vtkActor2D()
            labelsActor.SetMapper(labelsMapper)

            self.vmtkRenderer.Renderer.AddActor(labelsActor)

            surfaceMapper = vtk.vtkPolyDataMapper()
            surfaceMapper.SetInputData(self.Surface)
            surfaceMapper.ScalarVisibilityOff()
            surfaceActor = vtk.vtkActor()
            surfaceActor.SetMapper(surfaceMapper)
            surfaceActor.GetProperty().SetOpacity(0.25)

            self.vmtkRenderer.Renderer.AddActor(surfaceActor)

            #self.vmtkRenderer.Render()
            #self.vmtkRenderer.Renderer.RemoveActor(labelsActor)
            #self.vmtkRenderer.Renderer.RemoveActor(surfaceActor)

            ok = False
            while not ok:
                labelString = self.InputText("Please input boundary ids: ",self.LabelValidator)
                labels = [int(label) for label in labelString.split()]
                ok = True
                for label in labels:
                    if label not in list(range(numberOfBoundaries)):
                        ok = False

            for label in labels:
                boundaryIds.InsertNextId(label)

        if self.Method == 'simple':
            capper = vtkvmtk.vtkvmtkSimpleCapPolyData()
            capper.SetInputData(self.Surface)

        elif self.Method == 'centerpoint':
            capper = vtkvmtk.vtkvmtkCapPolyData()
            capper.SetInputData(self.Surface)
            capper.SetDisplacement(0.0)
            capper.SetInPlaneDisplacement(0.0)

        elif self.Method == 'smooth':
            triangle = vtk.vtkTriangleFilter()
            triangle.SetInputData(self.Surface)
            triangle.PassLinesOff()
            triangle.PassVertsOff()
            triangle.Update()
            capper = vtkvmtk.vtkvmtkSmoothCapPolyData()
            capper.SetInputConnection(triangle.GetOutputPort())
            capper.SetConstraintFactor(self.ConstraintFactor)
            capper.SetNumberOfRings(self.NumberOfRings)

        elif self.Method == 'annular':
            capper = vtkvmtk.vtkvmtkAnnularCapPolyData()
            capper.SetInputData(self.Surface)

        elif self.Method == 'concaveannular':
            from vmtk import vtkvmtkcontrib
            capper = vtkvmtkcontrib.vtkvmtkConcaveAnnularCapPolyData()
            capper.SetInputData(self.Surface)

        if self.Interactive:
            capper.SetBoundaryIds(boundaryIds)
        capper.SetCellEntityIdsArrayName(self.CellEntityIdsArrayName)
        capper.SetCellEntityIdOffset(self.CellEntityIdOffset)
        capper.Update()
        self.Surface = capper.GetOutput()

        if self.TriangleOutput == 1:
            triangle = vtk.vtkTriangleFilter()
            triangle.SetInputData(self.Surface)
            triangle.PassLinesOff()
            triangle.PassVertsOff()
            triangle.Update()
            self.Surface = triangle.GetOutput()

        # Since VTK 9.4, vtkPolyDataNormals passes pre-existing normals through
        # unchanged; strip inherited normals so they are always recomputed with
        # consistent outward orientation on the capped surface.
        cappedSurface = vtk.vtkPolyData()
        cappedSurface.ShallowCopy(self.Surface)
        cappedSurface.GetPointData().SetNormals(None)
        cappedSurface.GetCellData().SetNormals(None)

        normals = vtk.vtkPolyDataNormals()
        normals.SetInputData(cappedSurface)
        normals.AutoOrientNormalsOn()
        normals.SplittingOff()
        normals.ConsistencyOn()
        normals.Update()
        self.Surface = normals.GetOutput()


if __name__=='__main__':

    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtksurfacetonumpy.py,v $
## Language:  Python
## Date:      June 10, 2017
## Version:   1.4

##   Copyright (c) Richard Izzo, Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## Note: this class was contributed by
##       Richard Izzo (Github @rlizzo)
##       University at Buffalo
##       The Jacobs Institute

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
from vtk.numpy_interface import dataset_adapter as dsa
import sys

from vmtk import vtkvmtk
from vmtk import vmtkrenderer
from vmtk import pypes

try:
    import numpy as np
except ImportError:
    raise ImportError('Unable to Import vmtksurfacetonumpy module, numpy is not installed')


# convenience class for nested dictionaries
class vividict(dict):
    def __missing__(self, key):
        value = self[key] = type(self)()
        return value


class vmtkSurfaceToNumpy(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Surface = None
        self.ConvertCellToPoint = 0
        self.ArrayDict = vividict()

        self.SetScriptName('vmtkSurfaceToNumpy')
        self.SetScriptDoc('Takes a VTK triangulated surface vtkPolyData file (optionally containing point data scalar '
                          'arrays) and returns a nested python dictionary containing numpy arrays specifying vertex '
                          'points, associated scalar data, and cell data yielding triangle connectivity')
        self.SetInputMembers([
            ['Surface','i','vtkPolyData',1,'','the input surface','vmtksurfacereader'],
            ['ConvertCellToPoint', 'celltopoint', 'bool', 1, '', 'convert cell data to point data']])
        self.SetOutputMembers([
            ['ArrayDict','o','dict',1,'','the output dictionary','vmtknumpywriter']])

    def Execute(self):

        if self.Surface == None:
            self.PrintError('Error: No input surface.')

        if self.ConvertCellToPoint == 1:

            self.PrintLog('converting any cell data to point data')
            try:
                from vmtk import vmtksurfacecelldatatopointdata
            except ImportError:
                raise ImportError('unable to import vmtksurfacecelldatatopointdata module')
            surfaceCellToPoint = vmtksurfacecelldatatopointdata.vmtkSurfaceCellDataToPointData()
            surfaceCellToPoint.Surface = self.Surface
            surfaceCellToPoint.Execute()

            self.PrintLog('wrapping vtkPolyData object')
            surfWrapper = dsa.WrapDataObject(surfaceCellToPoint.Surface)

        else:
            self.PrintLog('wrapping vtkPolyData object')
            surfWrapper = dsa.WrapDataObject(self.Surface)

            self.PrintLog('converting cell data: ')
            for cellKey in surfWrapper.CellData.keys():
                self.PrintLog(cellKey)
                self.ArrayDict['CellData'][cellKey] = np.array(surfWrapper.CellData.GetArray(cellKey))

        self.PrintLog('converting points')
        self.ArrayDict['Points'] = np.array(surfWrapper.Points)

        self.PrintLog('converting point data: ')
        for pointKey in surfWrapper.PointData.keys():
            self.PrintLog(pointKey)
            self.ArrayDict['PointData'][pointKey] = np.array(surfWrapper.PointData.GetArray(pointKey))

        if not surfWrapper.PointData.keys():
            self.ArrayDict['PointData'] = {}

        self.PrintLog('converting cell connectivity list')
        numberOfCells = surfWrapper.VTKObject.GetNumberOfCells()
        numberOfPointsPerCell = surfWrapper.VTKObject.GetCell(0).GetNumberOfPoints()

        cellArray = np.zeros(shape=(numberOfCells, numberOfPointsPerCell), dtype=np.int32)
        it = np.nditer(cellArray, flags=['multi_index'], op_flags=['readwrite'])

        # this is an efficient ndarray iterator method. the loop "for x in it" pulls an element out of
        # the cellArray iterator (it) and with the elipses syntax (x[...]) writes the point id.
        # this is equivalent to writing
        # for cellId in range(numberOfCells):
        #     cell = surfWrapper.VTKObject.GetCell(cellId)
        #     for point in range(numberOfPointsPerCell):
        #         cellArray[cellId, point] = cell.GetPointId(point)

        for x in it:
            x[...] = surfWrapper.VTKObject.GetCell(it.multi_index[0]).GetPointId(it.multi_index[1])

        self.ArrayDict['CellData']['CellPointIds'] = cellArray


if __name__=='__main__':
    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtksurfacemassproperties.py,v $
## Language:  Python
## Date:      $Date: 2006/07/17 09:53:14 $
## Version:   $Revision: 1.8 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import sys
import vtk

from vmtk import pypes


class vmtkSurfaceMassProperties(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Surface = None

        self.SurfaceArea = 0.0
        self.Volume = 0.0
        self.ShapeIndex = 0.0

        self.SetScriptName('vmtksurfacemassproperties')
        self.SetScriptDoc('compute the volume of a closed surface.')
        self.SetInputMembers([
            ['Surface','i','vtkPolyData',1,'','the input surface','vmtksurfacereader']
            ])
        self.SetOutputMembers([
            ['SurfaceArea','area','float',1,''],
            ['Volume','volume','float',1,''],
            ['ShapeIndex','shape','float',1,'']
            ])

    def Execute(self):

        if self.Surface == None:
            self.PrintError('Error: No input surface.')

        cleaner = vtk.vtkCleanPolyData()
        cleaner.SetInputData(self.Surface)
        cleaner.Update()

        triangleFilter = vtk.vtkTriangleFilter()
        triangleFilter.SetInputConnection(cleaner.GetOutputPort())
        triangleFilter.Update()

        massProps = vtk.vtkMassProperties()
        massProps.SetInputConnection(triangleFilter.GetOutputPort())
        massProps.Update()

        self.SurfaceArea = massProps.GetSurfaceArea()
        self.Volume = massProps.GetVolume()
        self.ShapeIndex = massProps.GetNormalizedShapeIndex()


if __name__=='__main__':

    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkimageotsuthresholds.py,v $
## Language:  Python
## Date:      $Date: 2006/07/17 09:53:14 $
## Version:   $Revision: 1.8 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
from vmtk import vtkvmtk
import sys

from vmtk import pypes


class vmtkImageOtsuThresholds(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Image = None

        self.NumberOfHistogramBins = 128
        self.NumberOfThresholds = 1
        self.LabelOffset = 0
        self.OutputThresholdId = 0
        self.Thresholds = None
        self.Threshold = None

        self.SetScriptName('vmtkimageotsuthresholds')
        self.SetScriptDoc('automatically label an image using multiple Otsu thresholding')
        self.SetInputMembers([
            ['Image','i','vtkImageData',1,'','the input image','vmtkimagereader'],
            ['NumberOfHistogramBins','bins','int',1,'(0,)','the number of bins to use for generating the image histogram'],
            ['NumberOfThresholds','thresholds','int',1,'(0,)','the number of intensity thresholds to generate for generating the labeled image'],
            ['OutputThresholdId','othresholdid','int',1,'(0,)','the id of the threshold to send in output'],
            ['LabelOffset','offset','int',1,'(0,)','the lowest label value to use while generating the labeled image']
            ])
        self.SetOutputMembers([
            ['Image','o','vtkImageData',1,'','the output image','vmtkimagewriter'],
            ['Thresholds','othresholds','vtkFloatArray',1,'','the computed thresholds'],
            ['Threshold','othreshold','float',1,'','the nth computed threshold, with n set to OutputThresholdId']
            ])

    def Execute(self):

        if self.Image == None:
            self.PrintError('Error: No input image.')

        cast = vtk.vtkImageCast()
        cast.SetInputData(self.Image)
        cast.SetOutputScalarTypeToFloat()
        cast.Update()

        otsuFilter = vtkvmtk.vtkvmtkOtsuMultipleThresholdsImageFilter()
        otsuFilter.SetInputConnection(cast.GetOutputPort())
        otsuFilter.SetNumberOfHistogramBins(self.NumberOfHistogramBins)
        otsuFilter.SetNumberOfThresholds(self.NumberOfThresholds)
        otsuFilter.SetLabelOffset(self.LabelOffset)
        otsuFilter.Update()

        self.Image = otsuFilter.GetOutput()
        self.Thresholds = otsuFilter.GetThresholds()
        self.Threshold = self.Thresholds.GetValue(self.OutputThresholdId)


if __name__=='__main__':

    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



#!/usr/bin/env python

## Program:   PypeS
## Module:    $RCSfile: pypescript.py,v $
## Language:  Python
## Date:      $Date: 2006/05/26 12:36:30 $
## Version:   $Revision: 1.18 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import sys
import string
import os.path
import importlib
from inspect import isclass


class pypeMember(object):

    def __init__(self,memberName,optionName,memberType,memberLength,memberRange='',memberDoc='',memberIO=''):

        self.MemberName = memberName
        self.OptionName = optionName
        self.MemberType = memberType
        self.MemberLength = memberLength
        self.MemberRange = memberRange
        self.MemberDoc = memberDoc
        self.MemberIO = memberIO

        self.MemberPipe = ''
        self.MemberValue = None
        self.ExplicitPipe = ''
        self.AutoPipe = 1
        self.Pushed = 0

    def IsInRange(self,value):
        if not self.MemberRange:
            return
        if self.MemberRange[0] == '[' and self.MemberRange[-1] == ']':
            parsedMemberRange = [eval(entry) for entry in self.MemberRange[1:-1].split(',')]
            if value in parsedMemberRange:
                return True
        if self.MemberRange[0] == '(' and self.MemberRange[-1] == ')':
            parsedMemberRange = [entry for entry in self.MemberRange[1:-1].split(',')]
            if len(parsedMemberRange) < 2:
                return False
            inRange = True
            if parsedMemberRange[0] and value < eval(parsedMemberRange[0]):
                inRange = False
            if parsedMemberRange[1] and value > eval(parsedMemberRange[1]):
                inRange = False
            return inRange
        return False

    def GetRangeEnumeration(self):
        if not self.MemberRange:
            return []
        if self.MemberRange[0] == '[' and self.MemberRange[-1] == ']':
            parsedMemberRange = [eval(entry) for entry in self.MemberRange[1:-1].split(',')]
            return parsedMemberRange
        return []

    def GetRangeValues(self):
        if not self.MemberRange:
            return []
        if self.MemberRange[0] == '(' and self.MemberRange[-1] == ')':
            parsedMemberRange = []
            for entry in self.MemberRange[1:-1].split(','):
                if entry != '':
                    entry = eval(entry)
                else:
                    entry = None
                parsedMemberRange.append(entry)
            if len(parsedMemberRange) == 2:
                parsedMemberRange.append(None)
            if len(parsedMemberRange) != 3:
                return []
            return parsedMemberRange
        return []

    def GetRangeRepresentation(self):
        if not self.MemberRange:
            return []
        enumeration = self.GetRangeEnumeration()
        values = self.GetRangeValues()
        if enumeration:
            return 'in ' + str(enumeration)
        if values:
            representation = ''
            if values[0] != None:
                representation += '>= ' + str(values[0])
            if values[1] != None:
                if representation:
                    representation += ' and '
                representation += '<= ' + str(values[1])
            return representation


class pypeScript(object):
    '''the base class for every high-level script.

    It manages parsing, instantiates proper input/output methods for the
    script and keeps the script structure consistent. Each PypeScript is at
    the same time:
       - a script which can be called from the command line and piped to other scripts
       - a class which can be called from Python code (e.g. inside a tkinter GUI)

    Attributes:
        BuiltinOptionTypes (list):
        InputStream (function):
        OutputStream (function):
        ScriptName (string):
        ScriptDoc (string):
        Arguments (list):
        InputMembers (list):
        OutputMembers (list):
        Id (string):
        Disabled (bool):
        ExitOnError (bool):
        LogOn (bool):
        Progress (int):

    '''

    lastVisitedPath = '.'

    def __init__(self):

        self.BuiltinOptionTypes = ['int','str','float']

        self.InputStream = sys.stdin
        self.OutputStream = sys.stdout

        self.ScriptName = ''
        self.ScriptDoc = ''
        self.Arguments = []
        self.InputMembers = []
        self.OutputMembers = []
        self.Id = '0'
        self.Self = self
        self.Disabled = 0

        idMember = pypeMember('Id','id','str',1,'','script id')
        idMember.AutoPipe = 0
        self.InputMembers.append(idMember)
        self.OutputMembers.append(idMember)
        selfMember = pypeMember('Self','handle','self',1,'','handle to self')
        selfMember.AutoPipe = 0
        self.InputMembers.append(selfMember)
        self.OutputMembers.append(selfMember)
        selfMember = pypeMember('Disabled','disabled','bool',1,'','disable execution and piping')
        selfMember.AutoPipe = 0
        self.InputMembers.append(selfMember)

        self.ExitOnError = 1
        self.LogOn = 1

        self.Progress = 0

    def PrintLog(self,logMessage,indent=0):
        if not self.LogOn:
            return
        indentUnit = '    '
        indentation = ''
        for i in range(indent):
            indentation = indentation + indentUnit
        self.OutputStream.write(indentation + logMessage + '\n')

    def PrintError(self,errorMessage):
        self.OutputStream.write(errorMessage + '\n')
        if self.ExitOnError:
          self.Exit()
        else:
          raise RuntimeError(errorMessage)

    def Exit(self):
        sys.exit()

    def OutputText(self,text):
        self.OutputStream.write(text)

    def OutputProgress(self,progress,percentStep):
        if int(100 * progress)/percentStep == int(100 * self.Progress)/percentStep:
            return
        self.Progress = progress
        self.OutputStream.write('\r')
        self.OutputStream.write('Progress: '+str(int(100 * self.Progress))+'%')
        self.OutputStream.flush()
        self.InputInfo('Progress: '+str(int(100 * self.Progress))+'%')

    def EndProgress(self):
        self.OutputStream.write('\n')

    def InputInfo(self,prompt=''):
        self.OutputText(prompt)
        try:
            self.InputStream.prompt(prompt,info=True)
        except:
            pass

    def InputText(self,prompt='',validator=None):
        self.OutputText(prompt)
        try:
            self.InputStream.prompt(prompt)
        except:
            pass
        text = self.InputStream.readline()
        if text:
            text = text.rstrip('\n')
        if validator:
            while not validator(text):
                self.OutputText(prompt)
                try:
                    self.InputStream.prompt(prompt,info=False)
                except:
                    pass
                text = self.InputStream.readline()
                if text:
                    text = text.rstrip('\n')
        return text

    def PrintMembers(self,members):
        for memberEntry in members:
            memberName  = memberEntry.MemberName
            memberType = memberEntry.MemberType
            memberPipe = memberEntry.MemberPipe
            memberValue = self.__getattribute__(memberName)
##            if memberPipe:
##                self.__getattribute__("PrintLog")(memberName+' = '+memberPipe,1)
            if memberName == 'Self':
                pass
            elif (memberType in self.BuiltinOptionTypes + ["bool"]) or (memberType == 'list'):
                self.__getattribute__("PrintLog")(memberName+' = '+str(memberValue),1)
            elif memberValue:
                self.__getattribute__("PrintLog")(memberName+' = '+memberType,1)
            else:
                self.__getattribute__("PrintLog")(memberName+' = '+str(memberValue),1)

    def PrintInputMembers(self):
        self.PrintLog("Input " + self.ScriptName + " members:")
        self.PrintMembers(self.InputMembers)

    def PrintOutputMembers(self):
        self.PrintLog("Output " + self.ScriptName + " members:")
        self.PrintMembers(self.OutputMembers)

    def ConvertToPypeMembers(self,members):
        pypeMembers = []
        for member in members:
            if type(member) == pypeMember:
                pypeMembers.append(member)
            elif type(member) == list:
                pypeMembers.append(pypeMember(*member))
        return pypeMembers

    def SetInputMembers(self,membersToAppend):
        pypeMembersToAppend = self.ConvertToPypeMembers(membersToAppend)
        for member in pypeMembersToAppend:
            self.InputMembers.append(member)
            if member.MemberIO:
                filenameMember = pypeMember(self.GetIOInputFileNameMember(member.MemberName),self.GetIOFileNameOption(member.OptionName),'str',1,'','filename for the default ' + member.MemberName + ' reader')
##                filenameMember.AutoPipe = 0
                exec('self.'+filenameMember.MemberName+' = \'\'')
                self.InputMembers.append(filenameMember)

    def SetOutputMembers(self,membersToAppend):
        pypeMembersToAppend = self.ConvertToPypeMembers(membersToAppend)
        for member in pypeMembersToAppend:
            self.OutputMembers.append(member)
            if member.MemberIO:
                filenameMember = pypeMember(self.GetIOOutputFileNameMember(member.MemberName),self.GetIOFileNameOption(member.OptionName),'str',1,'','filename for the default ' + member.MemberName + ' writer')
##                filenameMember.AutoPipe = 0
                exec('self.'+filenameMember.MemberName+' = \'\'')
                self.InputMembers.append(filenameMember)

    def SetScriptName(self,scriptName):
        self.ScriptName = scriptName

    def SetScriptDoc(self,scriptDoc):
        self.ScriptDoc = scriptDoc

    def GetScriptDocString(self):
        return self.ScriptDoc

    def GetUsageString(self):
        usageString = ''
        scriptUsageString = os.path.splitext(self.ScriptName)[0]
        if self.ScriptDoc:
            scriptUsageString += ' : ' + self.ScriptDoc
        useTextWrap = 1
        try:
            import textwrap
        except ImportError:
            useTextWrap = 0
        else:
            textwrapper = textwrap.TextWrapper()
            textwrapper.initial_indent = ''
            textwrapper.subsequent_indent = ' '
        if useTextWrap:
            usageString += textwrapper.fill(scriptUsageString)
        else:
            usageString += scriptUsageString
        for memberList in [self.InputMembers, self.OutputMembers]:
            if memberList == self.InputMembers :
                 usageString += '\n' + '  ' + 'Input arguments:'
            elif memberList == self.OutputMembers :
                 usageString += '\n' + '  ' + 'Output arguments:'
            for memberEntry in memberList:
                memberUsageString = ''
                indentation = '   '
                if useTextWrap:
                    textwrapper.initial_indent = indentation
                    textwrapper.subsequent_indent = indentation + '  '
                memberName  = memberEntry.MemberName
                option = memberEntry.OptionName
                memberType = memberEntry.MemberType
                memberLength = memberEntry.MemberLength
                memberRange = memberEntry.MemberRange
                memberDoc = memberEntry.MemberDoc
                if option!='':
                    default = 0
                    if memberLength == 0:
                        memberUsageString += '-' + option + ' ' + memberName
                    elif memberType in self.BuiltinOptionTypes + ["bool"]:
                        default = self.__getattribute__(memberName)
                        memberUsageString += '-' + option + ' ' + memberName + ' (' + memberType + ',' + str(memberLength) + ')'
                        if memberRange:
                            memberUsageString += " " + memberEntry.GetRangeRepresentation()
                        if default != '':
                            memberUsageString += '; default=' + str(default)
                    else:
                        memberUsageString += '-' + option + ' ' + memberName + ' (' + memberType + ',' + str(memberLength) + ')'
                    if memberDoc != '':
                        memberUsageString += ': ' + memberDoc
                if useTextWrap:
                    usageString += '\n' + textwrapper.fill(memberUsageString)
                else:
                    usageString += '\n' + memberUsageString
        usageString += '\n'
        return usageString

    def GetHTMLUsageString(self):
        usageString = ''
        #usageString += '---' + '\n'
        #usageString += 'layout: default' + '\n'
        #usageString += '---' + '\n'
        usageString += '<h1>'
        usageString += self.ScriptName
        usageString += '</h1>'
        usageString += '\n'
        if self.ScriptDoc != '':
            usageString += '<h2>Description</h2>' + '\n'
            usageString += self.ScriptDoc + '\n'
        for memberList in [self.InputMembers, self.OutputMembers]:
            if memberList == self.InputMembers :
                 usageString += '<h3>Input arguments</h3>' + '\n'
            elif memberList == self.OutputMembers :
                 usageString += '<h3>Output arguments</h3>' + '\n'
            usageString += '<table class="vmtkscripts">' + '\n'
            usageString += '<tr>' + '\n'
            usageString += '<th>Argument</th><th>Variable</th><th>Type</th><th>Length</th><th>Range</th><th>Default</th><th>Description</th>\n'
            usageString += '</tr>' + '\n'
            for memberEntry in memberList:
                memberUsageString = ''
                memberName  = memberEntry.MemberName
                option = memberEntry.OptionName
                memberType = memberEntry.MemberType
                memberLength = memberEntry.MemberLength
                memberRange = memberEntry.MemberRange
                memberDoc = memberEntry.MemberDoc
                if option!='':
                    default = 0
                    if memberLength == 0:
                        memberUsageString += '<td>' + option + '</td><td>' + memberName + '</td><td></td><td></td><td></td><td></td>'
                    elif memberType in self.BuiltinOptionTypes + ["bool"]:
                        default = self.__getattribute__(memberName)
                        memberUsageString += '<td>' + option + '</td><td>' + memberName + '</td><td>' + memberType + '</td><td>' + str(memberLength) + '</td><td>' + str(memberRange) + '</td><td>' + str(default) + '</td>'
                    else:
                        memberUsageString += '<td>' + option + '</td><td>' + memberName + '</td><td>' + memberType + '</td><td>' + str(memberLength) + '</td><td></td><td></td>'
                    if not memberDoc:
                        memberDoc = ''
                    memberUsageString += '<td>' + memberDoc + '</td>'
                    #memberUsageString += ' | '
                memberUsageString += '\n'
                usageString += '<tr>' + memberUsageString + '</tr>' + '\n'
            usageString += '</table>\n'
        usageString += '\n'
        return usageString

    def GetDokuWikiUsageString(self):
        usageString = ''
        usageString = '======'
        usageString += self.ScriptName
        usageString += '======'
        usageString += '\n'
        if self.ScriptDoc != '':
            usageString += '=====Description=====' + '\n'
            usageString += self.ScriptDoc + '\n'
        for memberList in [self.InputMembers, self.OutputMembers]:
            if memberList == self.InputMembers :
                 usageString += '=====Input arguments=====' + '\n'
            elif memberList == self.OutputMembers :
                 usageString += '=====Output arguments=====' + '\n'
            usageString += '^ Argument ^ Variable ^ Type ^ Length ^ Range ^ Default ^ Description ^\n'
            for memberEntry in memberList:
                memberUsageString = ''
                memberName  = memberEntry.MemberName
                option = memberEntry.OptionName
                memberType = memberEntry.MemberType
                memberLength = memberEntry.MemberLength
                memberRange = memberEntry.MemberRange
                memberDoc = memberEntry.MemberDoc
                if option!='':
                    default = 0
                    if memberLength == 0:
                        memberUsageString += '| ' + option + ' | ' + memberName + ' | | | | | '
                    elif memberType in self.BuiltinOptionTypes + ["bool"]:
                        default = self.__getattribute__(memberName)
                        memberUsageString += '| ' + option + ' | ' + memberName + ' | ' + memberType + ' | ' + str(memberLength) + ' | ' + str(memberRange) + ' | ' + str(default) + ' | '
                    else:
                        memberUsageString += '| ' + option + ' | ' + memberName + ' | ' + memberType + ' | ' + str(memberLength) + ' | | | '
                    if memberDoc != '':
                        memberUsageString += memberDoc
                    memberUsageString += ' | '
                memberUsageString += '\n'
                usageString += memberUsageString
        usageString += '\n'
        return usageString

    def GetPmWikiUsageString(self):
        usageString = 'version=pmwiki-2.2.0-beta65 ordered=1 urlencoded=1\n'
        usageString += 'name=VmtkScripts.' + self.ScriptName.capitalize() + '\n'
        usageString += 'text='
        usageString += '!' + self.ScriptName
        usageString += '%0a'
        if self.ScriptDoc != '':
            usageString += '!!Description' + '%0a'
            usageString += self.ScriptDoc + '%0a'
            usageString += '%0a'
        for memberList in [self.InputMembers, self.OutputMembers]:
            if memberList == self.InputMembers :
                 usageString += '!!Input arguments' + '%0a'
            elif memberList == self.OutputMembers :
                 usageString += '!!Output arguments' + '%0a'
            usageString += '||border=1 cellpadding=3 cellspacing=0' + '%0a'
            usageString += '||!Argument ||!Variable ||!Type ||!Length ||!Range ||!Default ||!Description ||%0a'
            for memberEntry in memberList:
                memberUsageString = ''
                memberName  = memberEntry.MemberName
                option = memberEntry.OptionName
                memberType = memberEntry.MemberType
                memberLength = memberEntry.MemberLength
                memberRange = memberEntry.MemberRange
                memberDoc = memberEntry.MemberDoc
                if option!='':
                    default = 0
                    if memberLength == 0:
                        memberUsageString += '||' + option + ' ||' + memberName + ' || || || || ||'
                    elif memberType in self.BuiltinOptionTypes + ["bool"]:
                        default = self.__getattribute__(memberName)
                        memberUsageString += '||' + option + ' ||' + memberName + ' ||' + memberType + ' ||' + str(memberLength) + ' ||' + str(memberRange) + ' ||' + str(default) + ' ||'
                    else:
                        memberUsageString += '||' + option + ' ||' + memberName + ' ||' + memberType + ' ||' + str(memberLength) + ' || || ||'
                    if memberDoc != '':
                        memberUsageString += memberDoc
                    memberUsageString += ' ||'
                memberUsageString += '%0a'
                usageString += memberUsageString
            usageString += '%0a'
        usageString += '%0a'
        return usageString

    def ParseArguments(self):
        for arg in self.Arguments:
            if arg == '--help':
                self.PrintLog('')
                self.OutputText(self.GetUsageString())
                self.PrintLog('')
                return 0
            if arg == '--doc':
                self.PrintLog('')
                self.OutputText(self.GetScriptDocString())
                self.PrintLog('')
                return 0
            if arg == '--html':
                self.PrintLog('')
                self.OutputText(self.GetHTMLUsageString())
                self.PrintLog('')
                return 0
            if arg == '--dokuwiki':
                self.PrintLog('')
                self.OutputText(self.GetDokuWikiUsageString())
                self.PrintLog('')
                return 0
            if arg == '--pmwiki':
                self.PrintLog('')
                self.OutputText(self.GetPmWikiUsageString())
                self.PrintLog('')
                return 0
            if (arg[0] == '-') & (len(arg)==1):
                self.PrintError(self.GetUsageString() + '\n' + self.ScriptName + ' error: unknown option ' + arg + '\n')
                return 0
            if (arg[0] == '-'):
                if (arg[1] in string.ascii_letters):
                    matchingMembers = [member for member in self.InputMembers if member.OptionName in [arg.lstrip('-'), arg.lstrip('-').rstrip('@')]]
                    if not matchingMembers:
                        self.PrintError(self.GetUsageString() + '\n' + self.ScriptName + ' error: unknown option ' + arg + '\n')
                        return 0

        for memberEntry in self.InputMembers:

            if not memberEntry.OptionName:
                continue

            memberName  = memberEntry.MemberName
            option = '-' + memberEntry.OptionName
            memberType = memberEntry.MemberType
            memberLength = memberEntry.MemberLength
            memberRange = memberEntry.MemberRange
            memberValues = []
            activated = 0
            explicitPipe = 0

            specifiedOptions = [arg for arg in self.Arguments if (arg[0] == '-') and (arg[1] in string.ascii_letters + '-')]

            pushedOption = option + '@'
            if pushedOption in specifiedOptions:
                memberEntry.Pushed = 1
                specifiedOptions[specifiedOptions.index(pushedOption)] = option
                self.Arguments[self.Arguments.index(pushedOption)] = option

            if option in specifiedOptions:
                if memberLength == 0:
                    activated = 1
                optionValues = []
                optionIndex = self.Arguments.index(option)
                if option != specifiedOptions[-1]:
                    nextOptionIndex = self.Arguments.index(specifiedOptions[specifiedOptions.index(option)+1])
                    optionValues = self.Arguments[optionIndex+1:nextOptionIndex]
                else:
                    optionValues = self.Arguments[optionIndex+1:]
                for value in optionValues:
                    if value[0] == '@':
                        memberEntry.ExplicitPipe = value[1:]
                        if value[1:] == '':
                            memberEntry.ExplicitPipe = 'None'
                    else:
                        if memberType in self.BuiltinOptionTypes:
                            exec('castValue = '+memberType+'(\''+value+'\')', globals())
                            memberValues.append(castValue)
                        elif memberType == 'bool':
                            exec('castValue = int(\''+value+'\')', globals())
                            memberValues.append(castValue)
                        else:
                            memberValues.append(value)
            else:
                continue

            if memberLength != -1:
                if (len(memberValues) != memberLength) and not memberEntry.ExplicitPipe:
                    self.PrintError(self.GetUsageString() + '\n' + 'Error for option '+option+': '+str(len(memberValues))+' entries given, '+str(memberLength)+' expected.' + '\n')
                    return 0

            if len(memberValues) > 0:
                if (memberLength==0):
                    if (activated == 1):
                        setattr(self,memberName,1)
                        memberEntry.MemberValue = 1
                elif (memberLength==1):
                    setattr(self,memberName,memberValues[0])
                    memberEntry.MemberValue = memberValues[0]
                else:
                    setattr(self,memberName,memberValues)
                    memberEntry.MemberValue = memberValues

            if memberType == 'bool':
                if [value for value in memberValues if value not in [0,1]]:
                    self.PrintError(self.GetUsageString() + '\n' + 'Error for option '+option+': should be either 0 or 1' + '\n')
                    return 0

            if memberRange != '':
                if [value for value in memberValues if not memberEntry.IsInRange(value)]:
                    self.PrintError(self.GetUsageString() + '\n' + 'Error for option '+option+': should be ' + memberEntry.GetRangeRepresentation() + '\n')
                    return 0

            if (memberType != ''):
                if (memberLength==0):
                    if activated:
                        self.PrintLog(memberName + ' = ' + 'on',1)
                    else:
                        self.PrintLog(memberName + ' = ' + 'off',1)
                else:
                    if (memberLength==-1):
                        memberLength = len(memberValues)
                    if memberEntry.ExplicitPipe:
                        self.PrintLog(memberName+ ' = @' + memberEntry.ExplicitPipe,1)
                    else:
                        self.PrintLog(memberName+ ' = ' + str(self.__getattribute__(memberName)),1)
        return 1

    def IORead(self):
        try:
            from vmtk import pypes
        except ImportError:
            return None
        for member in self.InputMembers:
            if member.MemberIO:
                filename = eval('self.' + self.GetIOInputFileNameMember(member.MemberName))
                if filename:
                    try:
                        readerModule = importlib.import_module('vmtk.'+member.MemberIO)
                        # Find the principle class to instantiate the requested action defined inside the requested writerModule script.
                        # Returns a single member list (containing the principle class name) which satisfies the following criteria:
                        #   1) is a class defined within the script
                        #   2) the class is a subclass of pypes.pypescript
                        readerModuleClasses = [x for x in dir(readerModule) if isclass(getattr(readerModule, x)) and issubclass(getattr(readerModule, x), pypes.pypeScript)]
                        readerModuleClassName = readerModuleClasses[0]
                    except ImportError:
                        self.PrintError('Cannot import module ' + member.MemberIO + ' required for reading ' + member.MemberName)
                    reader = getattr(readerModule, readerModuleClassName)
                    reader = reader()
                    reader.InputFileName = filename
                    reader.LogOn = self.LogOn
                    reader.InputStream = self.InputStream
                    reader.OutputStream = self.OutputStream
                    reader.Execute()
                    exec('self.' + member.MemberName + ' = reader.Output')

    def IOWrite(self):
        try:
            from vmtk import pypes
        except ImportError:
            return None
        for member in self.OutputMembers:
            if member.MemberIO:
                filename = eval('self.' + self.GetIOOutputFileNameMember(member.MemberName))
                writerInput = eval('self.' + member.MemberName)
                if filename:
                    # Create code object representing local import of requested IO script
                    try:
                        writerModule = importlib.import_module('vmtk.'+member.MemberIO)
                        # Find the principle class to instantiate the requested action defined inside the requested writerModule script.
                        # Returns a single member list (containing the principle class name) which satisfies the following criteria:
                        #   1) is a class defined within the script
                        #   2) the class is a subclass of pypes.pypescript
                        writerModuleClasses = [x for x in dir(writerModule) if isclass(getattr(writerModule, x)) and issubclass(getattr(writerModule, x), pypes.pypeScript)]
                        writerModuleClassName = writerModuleClasses[0]
                    except ImportError:
                        self.PrintError('Cannot import module ' + member.MemberIO + ' required for writing ' + member.MemberIO)
                    writer = getattr(writerModule, writerModuleClassName)
                    writer = writer()
                    writer.Input = writerInput
                    writer.OutputFileName = filename
                    writer.LogOn = self.LogOn
                    writer.InputStream = self.InputStream
                    writer.OutputStream = self.OutputStream
                    writer.Execute()

    def GetIOInputFileNameMember(self,memberName):
        return memberName + 'InputFileName'

    def GetIOOutputFileNameMember(self,memberName):
        return memberName + 'OutputFileName'

    def GetIOFileNameOption(self,optionName):
        return optionName + 'file'

    def Execute(self):
        pass

    def Deallocate(self):
        pass


class pypeMain(object):

    def __init__(self):
        self.Arguments = None

    def Execute(self):
        from vmtk import pype
        pipe = pype.Pype()
        pipe.Arguments = self.Arguments
        pipe.ParseArguments()
        pipe.Execute()



## Module:    $RCSfile: vmtknumpytosurface.py,v $
## Language:  Python
## Date:      June 10, 2017
## Version:   1.4

##   Copyright (c) Richard Izzo, Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## Note: this class was contributed by
##       Richard Izzo (Github @rlizzo)
##       University at Buffalo
##       The Jacobs Institute

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
import sys

from vmtk import vtkvmtk
from vmtk import vmtkrenderer
from vmtk import pypes

try:
    import numpy as np
except ImportError:
    raise ImportError('Unable to Import vmtknumpytosurface module, numpy is not installed')


class vmtkNumpyToSurface(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.ArrayDict = None

        self.SetScriptName('vmtkNumpyToSurface')
        self.SetScriptDoc('Takes a nested python dictionary containing numpy arrays specifying vertex '
                          'points, associated scalar data, and cell data specifying triangle connectivity'
                          'and returns a VTK triangulated surface vtkPolyData file')
        self.SetInputMembers([
            ['ArrayDict','i','dict',1,'','the input array dictionary','vmtknumpyreader']])
        self.SetOutputMembers([
            ['Surface','o','vtkPolyData',1,'','the output surface','vmtksurfacewriter']])

    def Execute(self):

        polyData = vtk.vtkPolyData()

        self.PrintLog('converting points')
        points = vtk.vtkPoints()
        for xyzPoint in self.ArrayDict['Points']:
            points.InsertNextPoint(xyzPoint)

        polyData.SetPoints(points)

        self.PrintLog('converting numpy array to surface')
        for pointKey in self.ArrayDict['PointData'].keys():

            if np.issubdtype(self.ArrayDict['PointData'][pointKey].dtype, np.floating):
                pointDataArray = vtk.vtkFloatArray()
            else:
                for checkDt in [np.integer, np.uint8, np.uint16, np.uint32, np.uint64]:
                    if np.issubdtype(self.ArrayDict['PointData'][pointKey].dtype, checkDt):
                        pointDataArray = vtk.vtkIntArray()
                        break
                    else:
                        continue

            try:
                pointDataComponents = self.ArrayDict['PointData'][pointKey].shape[1]
            except IndexError:
                pointDataComponents = 1

            pointDataArray.SetNumberOfComponents(pointDataComponents)
            pointDataArray.SetName(pointKey)

            if pointDataComponents == 1:
                pointDataArray.SetNumberOfValues(self.ArrayDict['PointData'][pointKey].size)
                for index, pointData in enumerate(self.ArrayDict['PointData'][pointKey]):
                    pointDataArray.SetValue(index, pointData)
                polyData.GetPointData().SetActiveScalars(pointKey)
                polyData.GetPointData().SetScalars(pointDataArray)
            else:
                for pointData in self.ArrayDict['PointData'][pointKey]:
                    pointDataArray.InsertNextTuple(pointData)
                polyData.GetPointData().SetActiveVectors(pointKey)
                polyData.GetPointData().SetVectors(pointDataArray)

        def mkVtkIdList(it):
            vil = vtk.vtkIdList()
            for i in it:
                vil.InsertNextId(int(i))
            return vil

        for cellKey in self.ArrayDict['CellData'].keys():

            if cellKey == 'CellPointIds':
                cellDataArray = vtk.vtkCellArray()
                for cellId in self.ArrayDict['CellData'][cellKey]:
                    if type(cellId) is np.ndarray:
                        cellDataArray.InsertNextCell(mkVtkIdList(cellId))
                    else:
                        for cell in cellId:
                            cellDataArray.InsertNextCell(mkVtkIdList(cell))
                polyData.SetPolys(cellDataArray)

            else:

                if np.issubdtype(self.ArrayDict['CellData'][cellKey].dtype, np.floating):
                    cellDataArray = vtk.vtkFloatArray()
                if np.issubdtype(self.ArrayDict['CellData'][cellKey].dtype, np.integer):
                    cellDataArray = vtk.vtkIntArray()

                try:
                    cellDataComponents = self.ArrayDict['CellData'][cellKey].shape[1]
                except IndexError:
                    cellDataComponents = 1

                cellDataArray.SetNumberOfComponents(pointDataComponents)
                cellDataArray.SetName(cellKey)

                if cellDataComponents == 1:
                    cellDataArray.SetNumberOfValues(self.ArrayDict['CellData'][cellKey].size)
                    for index, cellData in enumerate(self.ArrayDict['CellData'][cellKey]):
                        cellDataArray.SetValue(index, cellData)
                    polyData.GetCellData().SetActiveScalars(cellKey)
                    polyData.GetCellData().SetScalars(cellDataArray)

                else:
                    for cellData in self.ArrayDict['CellData'][cellKey]:
                        pointDataArray.InsertNextTuple(cellData)
                    polyData.GetCellData().SetActiveVectors(cellKey)
                    polyData.GetCellData().SetVectors(cellDataArray)

        self.Surface = polyData


if __name__=='__main__':
    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtksurfacescaling.py,v $
## Language:  Python
## Date:      $Date: 2005/09/14 09:49:59 $
## Version:   $Revision: 1.4 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
import sys

from vmtk import pypes


class vmtkSurfaceScaling(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Surface = None

        self.ScaleFactor = None
        self.ScaleFactorX = None
        self.ScaleFactorY = None
        self.ScaleFactorZ = None

        self.SetScriptName('vmtksurfacescaling')
        self.SetScriptDoc('scale a surface by an isotropic factor, or x,y,z directions by separate factors')
        self.SetInputMembers([
            ['Surface','i','vtkPolyData',1,'','the input surface','vmtksurfacereader'],
            ['ScaleFactor','scale','float',1,'(0.0,)','isotropic scaling factor'],
            ['ScaleFactorX','scalex','float',1,'(0.0,)','scaling factor in x direction'],
            ['ScaleFactorY','scaley','float',1,'(0.0,)','scaling factor in y direction'],
            ['ScaleFactorZ','scalez','float',1,'(0.0,)','scaling factor in z direction'],
            ])
        self.SetOutputMembers([
            ['Surface','o','vtkPolyData',1,'','the output surface','vmtksurfacewriter']
            ])

    def Execute(self):

        if (self.Surface == None):
            self.PrintError('Error: no Surface.')

        transform = vtk.vtkTransform()
        s = self.ScaleFactor or 1.0
        x = self.ScaleFactorX or 1.0
        y = self.ScaleFactorY or 1.0
        z = self.ScaleFactorZ or 1.0
        transform.Scale(s*x, s*y, s*z)

        transformFilter = vtk.vtkTransformPolyDataFilter()
        transformFilter.SetInputData(self.Surface)
        transformFilter.SetTransform(transform)
        transformFilter.Update()

        self.Surface = transformFilter.GetOutput()


if __name__=='__main__':
    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkmeshtransform.py,v $
## Language:  Python
## Date:      $Date: 2005/09/14 09:49:59 $
## Version:   $Revision: 1.4 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## Note: this class was improved by
##       Hugo Gratama van Andel
##       Academic Medical Centre - University of Amsterdam
##       Dept. Biomedical Engineering  & Physics

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
import sys

from vmtk import pypes


class vmtkMeshTransform(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Mesh = None

        self.MatrixCoefficients = []
        self.InvertMatrix = 0
        self.Matrix4x4 = None
        self.Rotation = [0.0,0.0,0.0]
        self.Translation = [0.0,0.0,0.0]
        self.Scaling = [1.0,1.0,1.0]

        #TODO: define covariant vector array names

        self.SetScriptName('vmtkmeshtransform')
        self.SetScriptDoc('transform a mesh with a provided matrix')
        self.SetInputMembers([
            ['Mesh','i','vtkPolyData',1,'','the input mesh','vmtkmeshreader'],
            ['Matrix4x4','matrix4x4','vtkMatrix4x4',1,'','the input transform matrix'],
            ['MatrixCoefficients','matrix','float',16,'','coefficients of transform matrix'],
            ['InvertMatrix','invert','bool',1,'','invert matrix before applying transformation'],
            ['Rotation','rotation','float',3,'','rotations around the x-,y- and z-axis'],
            ['Translation','translation','float',3,'','translation in the x-,y- and z-directions'],
            ['Scaling','scaling','float',3,'','scaling of the x-,y- and z-directions']
            ])
        self.SetOutputMembers([
            ['Mesh','o','vtkPolyData',1,'','the output mesh','vmtkmeshwriter']
            ])

    def Execute(self):

        if (self.Mesh == None):
            self.PrintError('Error: no Mesh.')

        if not self.Matrix4x4:
            self.Matrix4x4 = vtk.vtkMatrix4x4()
            if self.MatrixCoefficients != []:
                self.PrintLog('Setting up transform matrix using specified coefficients')
                self.Matrix4x4.DeepCopy(self.MatrixCoefficients)
            elif self.Translation != [0.0,0.0,0.0] or self.Rotation != [0.0,0.0,0.0] or self.Scaling != [1.0,1.0,1.0]:
                self.PrintLog('Setting up transform matrix using specified translation, rotation and/or scaling')
                transform = vtk.vtkTransform()
                transform.RotateX(self.Rotation[0])
                transform.RotateY(self.Rotation[1])
                transform.RotateZ(self.Rotation[2])
                transform.Translate(self.Translation[0], self.Translation[1], self.Translation[2])
                transform.Scale(self.Scaling[0], self.Scaling[1], self.Scaling[2])
                self.Matrix4x4.DeepCopy(transform.GetMatrix())

        if self.InvertMatrix:
            self.Matrix4x4.Invert()

        transform = vtk.vtkMatrixToLinearTransform()
        transform.SetInput(self.Matrix4x4)

        transformFilter = vtk.vtkTransformFilter()
        transformFilter.SetInputData(self.Mesh)
        transformFilter.SetTransform(transform)
        transformFilter.Update()

        self.Mesh = transformFilter.GetOutput()


if __name__=='__main__':
    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()
